@@ -45,7 +45,6 @@ var Zotero_Browser = new function() {
 	this.toggleMode = toggleMode;
 	this.toggleCollapsed = toggleCollapsed;
 	this.chromeLoad = chromeLoad;
-	this.chromeUnload = chromeUnload;
 	this.contentLoad = contentLoad;
 	this.contentHide = contentHide;
 	this.tabClose = tabClose;
@@ -59,7 +58,6 @@ var Zotero_Browser = new function() {
 	this.statusImage = null;
 	this.isScraping = false;
 	
-	var _scrapePopupShowing = false;
 	var _browserData = new Object();
 	
 	var _blacklist = [
@@ -110,15 +108,24 @@ var Zotero_Browser = new function() {
 		
 		window.addEventListener("load",
 			function(e) { Zotero_Browser.chromeLoad(e) }, false);
-		window.addEventListener("unload",
-			function(e) { Zotero_Browser.chromeUnload(e) }, false);
+		
+		ZoteroPane_Local.addReloadListener(reload);
+		reload();
+	}
+	
+	/**
+	 * Called when Zotero is reloaded
+	 */
+	function reload() {
+		// Handles the display of a div showing progress in scraping
+		Zotero_Browser.progress = new Zotero.ProgressWindow();
 	}
 	
 	/**
 	 * Scrapes a page (called when the capture icon is clicked
 	 * @return	void
 	 */
-	function scrapeThisPage(/*libraryID, collectionID*/) {
+	function scrapeThisPage(translator) {
 		if (Zotero.locked) {
 			Zotero_Browser.progress.changeHeadline(Zotero.getString("ingester.scrapeError"));
 			var desc = Zotero.localeJoin([
@@ -129,13 +136,12 @@ var Zotero_Browser = new function() {
 			Zotero_Browser.progress.show();
 			Zotero_Browser.progress.startCloseTimer(8000);
 			return;
-			return;
 		}
 		
 		if (!Zotero.stateCheck()) {
 			Zotero_Browser.progress.changeHeadline(Zotero.getString("ingester.scrapeError"));
 			var desc = Zotero.getString("ingester.scrapeErrorDescription.previousError")
-				+ ' ' + Zotero.getString("general.restartFirefoxAndTryAgain");
+				+ ' ' + Zotero.getString("general.restartFirefoxAndTryAgain", Zotero.appName);
 			Zotero_Browser.progress.addDescription(desc);
 			Zotero_Browser.progress.show();
 			Zotero_Browser.progress.startCloseTimer(8000);
@@ -143,16 +149,25 @@ var Zotero_Browser = new function() {
 		}
 		
 		// get libraryID and collectionID
-		var libraryID, collectionID;
-		if(ZoteroPane) {
-			libraryID = ZoteroPane.getSelectedLibraryID();
-			collectionID = ZoteroPane.getSelectedCollection(true);
-		} else {
-			libraryID = collectionID = null;
+		var libraryID = null, collectionID = null;
+		if(ZoteroPane && !Zotero.isConnector) {
+			try {
+				if (!ZoteroPane.collectionsView.editable) {
+					Zotero_Browser.progress.changeHeadline(Zotero.getString("ingester.scrapeError"));
+					var desc = Zotero.getString('save.error.cannotMakeChangesToCollection');
+					Zotero_Browser.progress.addDescription(desc);
+					Zotero_Browser.progress.show();
+					Zotero_Browser.progress.startCloseTimer(8000);
+					return;
+				}
+				
+				libraryID = ZoteroPane.getSelectedLibraryID();
+				collectionID = ZoteroPane.getSelectedCollection(true);
+			} catch(e) {}
 		}
 		
 		// translate into specified library and collection
-		_getTabObject(this.tabbrowser.selectedBrowser).translate(libraryID, collectionID);
+		_getTabObject(Zotero_Browser.tabbrowser.selectedBrowser).translate(libraryID, collectionID, translator);
 	}
 	
 	/*
@@ -209,54 +224,6 @@ var Zotero_Browser = new function() {
 		tab.page.annotations.toggleCollapsed();
 	}
 	
-	/*
-	 * called to hide the collection selection popup
-	 */
-	function hidePopup(collectionID) {
-		_scrapePopupShowing = false;
-	}
-
-	/*
-	 * called to show the collection selection popup
-	 *
-	 * not currently used
-	 */
-	function showPopup(collectionID, parentElement) {
-		if(_scrapePopupShowing && parentElement.hasChildNodes()) {
-			return false;	// Don't dynamically reload popups that are already showing
-		}
-		_scrapePopupShowing = true;
-		parentElement.removeAllItems();
-		
-		if(collectionID == null) {	// show library
-			var newItem = document.createElement("menuitem");
-			newItem.setAttribute("label", Zotero.getString("pane.collections.library"));
-			newItem.setAttribute("class", "menuitem-iconic zotero-scrape-popup-library");
-			newItem.setAttribute("oncommand", 'Zotero_Browser.scrapeThisPage()');
-			parentElement.appendChild(newItem);
-		}
-		
-		var childrenList = Zotero.getCollections(collectionID);
-		for(var i = 0; i < childrenList.length; i++) {
-			if(childrenList[i].hasChildCollections()) {
-				var newItem = document.createElement("menu");
-				var subMenu = document.createElement("menupopup");
-				subMenu.setAttribute("onpopupshowing", 'Zotero_Browser.showPopup("'+childrenList[i].getID()+'", this)');
-				newItem.setAttribute("class", "menu-iconic zotero-scrape-popup-collection");
-				newItem.appendChild(subMenu);
-			} else {
-				var newItem = document.createElement("menuitem");
-				newItem.setAttribute("class", "menuitem-iconic zotero-scrape-popup-collection");
-			}
-			newItem.setAttribute("label", childrenList[i].getName());
-			newItem.setAttribute("oncommand", 'Zotero_Browser.scrapeThisPage("'+childrenList[i].getID()+'")');
-			
-			parentElement.appendChild(newItem);
-		}
-		
-		return true;
-	}
-	
 	/*
 	 * When chrome loads, register our event handlers with the appropriate interfaces
 	 */
@@ -277,11 +244,7 @@ var Zotero_Browser = new function() {
 				Zotero_Browser.updateStatus();
 			}, false);
 		// this is for pageshow, for updating the status of the book icon
-		this.appcontent.addEventListener("pageshow",
-			function(e) {
-				//Zotero.debug("pageshow");
-				Zotero_Browser.contentLoad(e);
-			}, true);
+		this.appcontent.addEventListener("pageshow", contentLoad, true);
 		// this is for turning off the book icon when a user navigates away from a page
 		this.appcontent.addEventListener("pagehide",
 			function(e) {
@@ -314,27 +277,18 @@ var Zotero_Browser = new function() {
 			function(e) { Zotero_Browser.resize(e) }, false);
 	}
 	
-	/*
-	 * Called when chrome is unloaded
-	 */
-	function chromeUnload() {
-	}
-	
 	/*
 	 * An event handler called when a new document is loaded. Creates a new document
 	 * object, and updates the status of the capture icon
 	 */
 	function contentLoad(event) {
-		var isHTML = event.originalTarget instanceof HTMLDocument;
 		var doc = event.originalTarget;
-		var rootDoc = doc;
+		var isHTML = doc instanceof HTMLDocument;
+		var rootDoc = (doc instanceof HTMLDocument ? doc.defaultView.top.document : doc);
+		var browser = Zotero_Browser.tabbrowser.getBrowserForDocument(rootDoc);
+		if(!browser) return;
 		
 		if(isHTML) {
-			// get the appropriate root document to check which browser we're on
-			while(rootDoc.defaultView.frameElement) {
-				rootDoc = rootDoc.defaultView.frameElement.ownerDocument;
-			}
-			
 			// ignore blacklisted domains
 			try {
 				if(doc.domain) {
@@ -361,27 +315,17 @@ var Zotero_Browser = new function() {
 		}
 		catch (e) {}
 		
-		// Figure out what browser this contentDocument is associated with
-		var browser;
-		for(var i=0; i<this.tabbrowser.browsers.length; i++) {
-			if(rootDoc == this.tabbrowser.browsers[i].contentDocument) {
-				browser = this.tabbrowser.browsers[i];
-				break;
-			}
-		}
-		if(!browser) return;
-		
 		// get data object
 		var tab = _getTabObject(browser);
 		
-		if(isHTML) {
+		if(isHTML && !Zotero.isConnector) {
 			var annotationID = Zotero.Annotate.getAnnotationIDFromURL(browser.currentURI.spec);
 			if(annotationID) {
 				if(Zotero.Annotate.isAnnotated(annotationID)) {
 					window.alert(Zotero.getString("annotations.oneWindowWarning"));
 				} else if(!tab.page.annotations) {
 					// enable annotation
-					tab.page.annotations = new Zotero.Annotations(this, browser, annotationID);
+					tab.page.annotations = new Zotero.Annotations(Zotero_Browser, browser, annotationID);
 					var saveAnnotations = function() {
 						tab.page.annotations.save();
 						tab.page.annotations = undefined;
@@ -395,42 +339,57 @@ var Zotero_Browser = new function() {
 		
 		// detect translators
 		tab.detectTranslators(rootDoc, doc);
+		
+		// register metadata updated event
+		if(isHTML) {
+			var contentWin = doc.defaultView;
+			if(!contentWin.haveZoteroEventListener) {
+				contentWin.addEventListener("ZoteroItemUpdated", itemUpdated, false);
+				contentWin.haveZoteroEventListener = true;
+			}
+		}
 	}
 
 	/*
 	 * called to unregister Zotero icon, etc.
 	 */
 	function contentHide(event) {
-		if(event.originalTarget instanceof HTMLDocument) {
-			// Get root document if this is a frameset
-			var doc = event.originalTarget;
-			var rootDoc = doc;
-			while(rootDoc.defaultView.frameElement) {
-				rootDoc = rootDoc.defaultView.frameElement.ownerDocument;
-			}
-			
-			// Figure out what browser this contentDocument is associated with
-			var browser;
-			for(var i=0; i<this.tabbrowser.browsers.length; i++) {
-				if(rootDoc == this.tabbrowser.browsers[i].contentDocument) {
-					browser = this.tabbrowser.browsers[i];
-					break;
-				}
-			}
-			
-			var tab = _getTabObject(browser);
-			if(!tab) return;
-			if(doc == tab.page.document || doc == rootDoc) {
-				// clear translator only if the page on which the pagehide event was called is
-				// either the page to which the translator corresponded, or the root document
-				// (the second check is probably paranoid, but won't hurt)
-				tab.clear();
-			}
-			
-			// update status
-			if(this.tabbrowser.selectedBrowser == browser) {
-				updateStatus();
-			}
+		var doc = event.originalTarget;
+		if(!(doc instanceof HTMLDocument)) return;
+	
+		var rootDoc = (doc instanceof HTMLDocument ? doc.defaultView.top.document : doc);
+		var browser = Zotero_Browser.tabbrowser.getBrowserForDocument(rootDoc);
+		if(!browser) return;
+		
+		var tab = _getTabObject(browser);
+		if(!tab) return;
+		
+		if(doc == tab.page.document || doc == rootDoc) {
+			// clear translator only if the page on which the pagehide event was called is
+			// either the page to which the translator corresponded, or the root document
+			// (the second check is probably paranoid, but won't hurt)
+			tab.clear();
+		}
+		
+		// update status
+		if(Zotero_Browser.tabbrowser.selectedBrowser == browser) {
+			updateStatus();
+		}
+	}
+	
+	/**
+	 * Called when item should be updated due to a DOM event
+	 */
+	function itemUpdated(event) {
+		try {
+		var doc = event.originalTarget;
+		var rootDoc = (doc instanceof HTMLDocument ? doc.defaultView.top.document : doc);
+		var browser = Zotero_Browser.tabbrowser.getBrowserForDocument(rootDoc);
+		var tab = _getTabObject(browser);
+		if(doc == tab.page.document || doc == rootDoc) tab.clear();
+		tab.detectTranslators(rootDoc, doc);
+		} catch(e) {
+			Zotero.debug(e);
 		}
 	}
 	
@@ -445,7 +404,6 @@ var Zotero_Browser = new function() {
 		tab.clear();
 		
 		// To execute if document object does not exist
-		_deleteTabObject(event.target.linkedBrowser);
 		toggleMode();
 	}
 	
@@ -472,6 +430,10 @@ var Zotero_Browser = new function() {
 			Zotero_Browser.statusImage.src = captureIcon;
 			Zotero_Browser.statusImage.tooltipText = tab.getCaptureTooltip();
 			Zotero_Browser.statusImage.hidden = false;
+			
+			Zotero_Browser.statusImage.addEventListener("load", function() {
+				document.getElementById("zotero-status-image-guidance").show();
+			}, false);
 		} else {
 			Zotero_Browser.statusImage.hidden = true;
 		}
@@ -508,16 +470,16 @@ var Zotero_Browser = new function() {
 	/*
 	 * Callback to be executed when an item has been finished
 	 */
-	function itemDone(obj, item, collection) {
-		var title = item.getField("title", false, true);
-		var icon = item.getImageSrc();
+	function itemDone(obj, dbItem, item, collection) {
+		var title = item.title;
+		var icon = Zotero.ItemTypes.getImageSrc(item.itemType);
 		Zotero_Browser.progress.show();
 		Zotero_Browser.progress.changeHeadline(Zotero.getString("ingester.scraping"));
 		Zotero_Browser.progress.addLines([title], [icon]);
 		
 		// add item to collection, if one was specified
 		if(collection) {
-			collection.addItem(item.getID());
+			collection.addItem(dbItem.id);
 		}
 		
 		if(Zotero_Browser.isScraping) {
@@ -530,58 +492,96 @@ var Zotero_Browser = new function() {
 		}
 	}
 	
+	/**
+	 * Called when status bar icon is right-clicked
+	 */
+	this.onStatusPopupShowing = function(e) {
+		var popup = e.target;
+		while(popup.hasChildNodes()) popup.removeChild(popup.lastChild);
+		
+		var tab = _getTabObject(this.tabbrowser.selectedBrowser);
+		var translators = tab.page.translators;
+		for(var i=0, n=translators.length; i<n; i++) {
+			let translator = translators[i];
+			
+			var menuitem = document.createElement("menuitem");
+			menuitem.setAttribute("label",
+				Zotero.getString("ingester.saveToZoteroUsing", translator.label));
+			menuitem.setAttribute("image", (translator.itemType === "multiple"
+				? "chrome://zotero/skin/treesource-collection.png"
+				: Zotero.ItemTypes.getImageSrc(translator.itemType)));
+			menuitem.setAttribute("class", "menuitem-iconic");
+			menuitem.addEventListener("command", function(e) {
+				scrapeThisPage(translator);
+			}, false);
+			popup.appendChild(menuitem);
+		}
+		
+		popup.appendChild(document.createElement("menuseparator"));
+		
+		var menuitem = document.createElement("menuitem");
+		menuitem.setAttribute("label", Zotero.getString("locate.libraryLookup.label"));
+		menuitem.setAttribute("tooltiptext", Zotero.getString("locate.libraryLookup.tooltip"));
+		menuitem.setAttribute("image", "chrome://zotero/skin/locate-library-lookup.png");
+		menuitem.setAttribute("class", "menuitem-iconic");
+		menuitem.addEventListener("command", _constructLookupFunction(tab, function(event, obj) {
+			var urls = [];
+			for each(var item in obj.newItems) {
+				var url = Zotero.OpenURL.resolve(item);
+				if(url) urls.push(url);
+			}
+			ZoteroPane.loadURI(urls, event);
+		}), false);
+		popup.appendChild(menuitem);		
+		
+		var locateEngines = Zotero.LocateManager.getVisibleEngines();
+		Zotero_LocateMenu.addLocateEngines(popup, locateEngines,
+			_constructLookupFunction(tab, function(e, obj) {
+				Zotero_LocateMenu.locateItem(e, obj.newItems);
+			}), true);
+	}
+	
+	
 	//////////////////////////////////////////////////////////////////////////////
 	//
 	// Private Zotero_Browser methods
 	//
 	//////////////////////////////////////////////////////////////////////////////
 	
+	function _constructLookupFunction(tab, success) {
+		return function(e) {
+			tab.page.translate.setTranslator(tab.page.translators[0]);
+			tab.page.translate.clearHandlers("done");
+			tab.page.translate.clearHandlers("itemDone");
+			tab.page.translate.setHandler("done", function(obj, status) {
+				if(status) {
+					success(e, obj);
+					Zotero_Browser.progress.close();
+				} else {
+					Zotero_Browser.progress.changeHeadline(Zotero.getString("ingester.lookup.error"));
+					Zotero_Browser.progress.startCloseTimer(8000);
+				}
+			});
+			
+			Zotero_Browser.progress.show();
+			Zotero_Browser.progress.changeHeadline(Zotero.getString("ingester.lookup.performing"));
+			tab.page.translate.translate(false);
+		}
+	}
+	
 	/*
 	 * Gets a data object given a browser window object
-	 * 
-	 * NOTE: Browser objects are associated with document objects via keys generated
-	 * from the time the browser object is opened. I'm not sure if this is the
-	 * appropriate mechanism for handling this, but it's what PiggyBank used and it
-	 * appears to work.
-	 *
-	 * Currently, the data object contains only one property: "translators," which
-	 * is an array of translators that should work with the given page as returned
-	 * from Zotero.Translate.getTranslator()
 	 */
 	function _getTabObject(browser) {
 		if(!browser) return false;
-		try {
-			var key = browser.getAttribute("zotero-key");
-			if(_browserData[key]) {
-				return _browserData[key];
-			}
-		} finally {
-			if(!key) {
-				var key = (new Date()).getTime();
-				browser.setAttribute("zotero-key", key);
-				return (_browserData[key] = new Zotero_Browser.Tab(browser));
-			}
+		if(!browser.zoteroBrowserData) {
+			browser.zoteroBrowserData = new Zotero_Browser.Tab(browser);
 		}
-		return false;
+		return browser.zoteroBrowserData;
 	}
 	
-	/*
-	 * Deletes the document object associated with a given browser window object
-	 */
-	function _deleteTabObject(browser) {
-		if(!browser) return false;
-		try {
-			var key = browser.getAttribute("zotero-key");
-			if(_browserData[key]) {
-				delete _browserData[key];
-				return true;
-			}
-		} finally {}
-		return false;
-	}
-	
-	/*
-	 * adds an annotation
+	/**
+	 * Adds an annotation
 	 */
 	 function _add(type, e) {
 		var tab = _getTabObject(Zotero_Browser.tabbrowser.selectedBrowser);
@@ -641,14 +641,7 @@ Zotero_Browser.Tab.prototype.clear = function() {
 /*
  * detects translators for this browser object
  */
-Zotero_Browser.Tab.prototype.detectTranslators = function(rootDoc, doc) {
-	// if there's already a scrapable page in the browser window, and it's
-	// still there, ensure it is actually part of the page, then return
-	if(this.page.translators && this.page.translators.length && this.page.document.location) {
-		if(this._searchFrames(rootDoc, this.page.document)) return;
-		this.clear();
-	}
-	
+Zotero_Browser.Tab.prototype.detectTranslators = function(rootDoc, doc) {	
 	if(doc instanceof HTMLDocument && doc.documentURI.substr(0, 6) != "about:") {
 		// get translators
 		var me = this;
@@ -656,7 +649,7 @@ Zotero_Browser.Tab.prototype.detectTranslators = function(rootDoc, doc) {
 		var translate = new Zotero.Translate.Web();
 		translate.setDocument(doc);
 		translate.setHandler("translators", function(obj, item) { me._translatorsAvailable(obj, item) });
-		translate.getTranslators();
+		translate.getTranslators(true);
 	} else if(doc.documentURI.substr(0, 7) == "file://") {
 		this._attemptLocalFileImport(doc);
 	}
@@ -720,7 +713,7 @@ Zotero_Browser.Tab.prototype._attemptLocalFileImport = function(doc) {
  * @param	{Integer}	libraryID
  * @param	{Integer}	collectionID
  */
-Zotero_Browser.Tab.prototype.translate = function(libraryID, collectionID) {
+Zotero_Browser.Tab.prototype.translate = function(libraryID, collectionID, translator) {
 	if(this.page.translators && this.page.translators.length) {
 		Zotero_Browser.progress.show();
 		Zotero_Browser.isScraping = true;
@@ -734,15 +727,14 @@ Zotero_Browser.Tab.prototype.translate = function(libraryID, collectionID) {
 		
 		var me = this;
 		
-		if(!this.page.hasBeenTranslated) {
-			// use first translator available
-			this.page.translate.setTranslator(this.page.translators[0]);
-			this.page.translate.setHandler("select", me._selectItems);
-			this.page.translate.setHandler("done", function(obj, item) { Zotero_Browser.finishScraping(obj, item) });
-			this.page.hasBeenTranslated = true;
-		}
+		// use first translator available
+		this.page.translate.setTranslator(translator ? translator : this.page.translators[0]);
+		
+		this.page.translate.clearHandlers("done");
 		this.page.translate.clearHandlers("itemDone");
-		this.page.translate.setHandler("itemDone", function(obj, item) { Zotero_Browser.itemDone(obj, item, collection) });
+		
+		this.page.translate.setHandler("done", function(obj, item) { Zotero_Browser.finishScraping(obj, item) });
+		this.page.translate.setHandler("itemDone", function(obj, dbItem, item) { Zotero_Browser.itemDone(obj, dbItem, item, collection) });
 		
 		this.page.translate.translate(libraryID);
 	}
@@ -755,12 +747,9 @@ Zotero_Browser.Tab.prototype.translate = function(libraryID, collectionID) {
 Zotero_Browser.Tab.prototype.getCaptureIcon = function() {
 	if(this.page.translators && this.page.translators.length) {
 		var itemType = this.page.translators[0].itemType;
-		if(itemType == "multiple") {
-			// Use folder icon for multiple types, for now
-			return "chrome://zotero/skin/treesource-collection.png";
-		} else {
-			return Zotero.ItemTypes.getImageSrc(itemType);
-		}
+		return (itemType === "multiple"
+				? "chrome://zotero/skin/treesource-collection.png"
+				: Zotero.ItemTypes.getImageSrc(itemType));
 	}
 	
 	return false;
@@ -784,7 +773,7 @@ Zotero_Browser.Tab.prototype.getCaptureTooltip = function() {
 /*
  * called when a user is supposed to select items
  */
-Zotero_Browser.Tab.prototype._selectItems = function(obj, itemList) {
+Zotero_Browser.Tab.prototype._selectItems = function(obj, itemList, callback) {
 	// this is kinda ugly, mozillazine made me do it! honest!
 	var io = { dataIn:itemList, dataOut:null }
 	var newDialog = window.openDialog("chrome://zotero/content/ingester/selectitems.xul",
@@ -794,7 +783,7 @@ Zotero_Browser.Tab.prototype._selectItems = function(obj, itemList) {
 		Zotero_Browser.progress.close();
 	}
 	
-	return io.dataOut;
+	callback(io.dataOut);
 }
 
 /*
@@ -802,17 +791,29 @@ Zotero_Browser.Tab.prototype._selectItems = function(obj, itemList) {
  */
 Zotero_Browser.Tab.prototype._translatorsAvailable = function(translate, translators) {
 	if(translators && translators.length) {
+		// if there's already a scrapable page in the browser window, and it's
+		// still there, ensure it is actually part of the page, then return
+		if(this.page.translators && this.page.translators.length && this.page.document.location) {
+			if(this.page.document.defaultView && !this.page.document.defaultView.closed) {
+				// if it is still there, switch translation to take place on 
+				if(!translators.length || this.page.translators[0].priority <= translators[0].priority) return;
+			} else {
+				this.clear();
+			}
+		}
+		
 		this.page.translate = translate;
 		this.page.translators = translators;
 		this.page.document = translate.document;
+	
+		this.page.translate.clearHandlers("select");
+		this.page.translate.setHandler("select", this._selectItems);
 	} else if(translate.type != "import" && translate.document.documentURI.length > 7
 			&& translate.document.documentURI.substr(0, 7) == "file://") {
 		this._attemptLocalFileImport(translate.document);
 	}
+	
 	Zotero_Browser.updateStatus();
 }
 
-// Handles the display of a div showing progress in scraping
-Zotero_Browser.progress = new Zotero.ProgressWindow();
-
 Zotero_Browser.init();
\ No newline at end of file
