@@ -192,30 +192,36 @@ var Zotero_File_Interface = new function() {
 		}
 	}
 	
-	/*
+	/**
 	 * Creates Zotero.Translate instance and shows file picker for file import
 	 */
-	function importFile() {
+	function importFile(file) {
 		var translation = new Zotero.Translate.Import();
-		var translators = translation.getTranslators();
-		
-		const nsIFilePicker = Components.interfaces.nsIFilePicker;
-		var fp = Components.classes["@mozilla.org/filepicker;1"]
-				.createInstance(nsIFilePicker);
-		fp.init(window, Zotero.getString("fileInterface.import"), nsIFilePicker.modeOpen);
-		
-		fp.appendFilters(nsIFilePicker.filterAll);
-		for(var i in translators) {
-			fp.appendFilter(translators[i].label, "*."+translators[i].target);
+		if(!file) {
+			var translators = translation.getTranslators();
+			
+			const nsIFilePicker = Components.interfaces.nsIFilePicker;
+			var fp = Components.classes["@mozilla.org/filepicker;1"]
+					.createInstance(nsIFilePicker);
+			fp.init(window, Zotero.getString("fileInterface.import"), nsIFilePicker.modeOpen);
+			
+			fp.appendFilters(nsIFilePicker.filterAll);
+			for(var i in translators) {
+				fp.appendFilter(translators[i].label, "*."+translators[i].target);
+			}
+			
+			var rv = fp.show();
+			if (rv !== nsIFilePicker.returnOK && rv !== nsIFilePicker.returnReplace) {
+				return false;
+			}
+			
+			file = fp.file;
 		}
 		
-		var rv = fp.show();
-		if (rv == nsIFilePicker.returnOK || rv == nsIFilePicker.returnReplace) {
-			translation.setLocation(fp.file);
-			// get translators again, bc now we can check against the file
-			translation.setHandler("translators", function(obj, item) { _importTranslatorsAvailable(obj, item) });
-			translators = translation.getTranslators();
-		}
+		translation.setLocation(file);
+		// get translators again, bc now we can check against the file
+		translation.setHandler("translators", function(obj, item) { _importTranslatorsAvailable(obj, item) });
+		translators = translation.getTranslators();
 	}
 	
 	
@@ -289,8 +295,11 @@ var Zotero_File_Interface = new function() {
 			Zotero_File_Interface.Progress.show(
 				Zotero.getString("fileInterface.itemsImported")
 			);
-			Zotero.DB.beginTransaction();
-			translation.translate();
+			
+			window.setTimeout(function() {
+				Zotero.DB.beginTransaction();
+				translation.translate();
+			}, 0);
 		} else {
 			// TODO: localize and remove fileInterface.fileFormatUnsupported string
 			var unsupportedFormat = "The selected file is not in a supported format.";
@@ -310,7 +319,7 @@ var Zotero_File_Interface = new function() {
 				null, null, {}
 			);
 			if (index == 1) {
-				window.loadURI("http://zotero.org/support/kb/importing");
+				ZoteroPane_Local.loadURI("http://zotero.org/support/kb/importing");
 			}
 		}
 	}
@@ -330,7 +339,7 @@ var Zotero_File_Interface = new function() {
 	 */
 	function _importDone(obj, worked) {
 		// add items to import collection
-		_importCollection.addItems(obj.newItems);
+		_importCollection.addItems([item.id for each(item in obj.newItems)]);
 		
 		Zotero.DB.commitTransaction();
 		
@@ -609,7 +618,34 @@ var Zotero_File_Interface = new function() {
 	 * Updates progress indicators based on current progress of translation
 	 */
 	this.updateProgress = function(translate) {
-		Zotero.updateZoteroPaneProgressMeter(translate.progress);
+		Zotero.updateZoteroPaneProgressMeter(translate.getProgress());
+		
+		var now = Date.now();
+		
+		// Don't repaint more than 10 times per second unless forced.
+		if(window.zoteroLastRepaint && (now - window.zoteroLastRepaint) < 100) return
+		
+		// Start a nested event queue
+		// TODO Remove when Fx > 14
+		var eventQueuePushed = "pushEventQueue" in Zotero.mainThread;
+		if(eventQueuePushed) {
+			Zotero.mainThread.pushEventQueue(null);
+		}
+		
+		try {
+			// Add the redraw event onto event queue
+			window.QueryInterface(Components.interfaces.nsIInterfaceRequestor)
+				.getInterface(Components.interfaces.nsIDOMWindowUtils)
+				.redraw();
+			
+			// Process redraw event
+			Zotero.mainThread.processNextEvent(false);
+		} finally {
+			// Close nested event queue
+			if(eventQueuePushed) Zotero.mainThread.popEventQueue();
+		}
+		
+		window.zoteroLastRepaint = now;
 	}
 }
 
