@@ -492,7 +492,7 @@ var Zotero_Citation_Dialog = new function () {
 		
 		// run preview function to re-sort, if it hasn't already been
 		// run
-		io.previewFunction();
+		io.sort();
 		
 		// add items back to list
 		scrollToItem = null;
@@ -565,7 +565,10 @@ var Zotero_Citation_Dialog = new function () {
 		if(_previewShown) {
 			document.documentElement.getButton("extra2").label = Zotero.getString("citation.hideEditor");		
 			if(text) {
-				editor.value = text;
+				io.preview(function(preview) {
+					_originalHTML = preview;
+					editor.value = text;
+				});
 			} else {
 				_updatePreview();
 			}
@@ -582,27 +585,23 @@ var Zotero_Citation_Dialog = new function () {
 		_getCitation();
 		Zotero.debug("got citation");
 		var isCustom = _previewShown && io.citation.citationItems.length	// if a citation is selected
+				&& _originalHTML
 				&& document.getElementById('editor').value != _originalHTML	// and citation has been edited
 		
 		if(isCustom) {	
 			var citation = document.getElementById('editor').value;
-		} else {
-			var citation = (io.citation.citationItems.length ? io.previewFunction() : "");
+			if(Zotero.Utilities.trim(citation) == "") {				
+				var promptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
+							.getService(Components.interfaces.nsIPromptService);
+				var insert = promptService.confirm(window,
+					Zotero.getString("integration.emptyCitationWarning.title"),
+					Zotero.getString("integration.emptyCitationWarning.body"));
+				if(!insert) return false;
+			}
+			io.citation.properties.custom = citation;
 		}
-		Zotero.debug("verified not custom");
-		
-		if(Zotero.Utilities.trim(citation) == "") {				
-			var promptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
-						.getService(Components.interfaces.nsIPromptService);
-			var insert = promptService.confirm(window,
-				Zotero.getString("integration.emptyCitationWarning.title"),
-				Zotero.getString("integration.emptyCitationWarning.body"));
-			if(!insert) return false;
-		}
-		Zotero.debug("verified not empty");
-		
-		if(isCustom) io.citation.properties.custom = citation;
 		
+		io.accept();
 		return true;
 	}
 	
@@ -611,6 +610,7 @@ var Zotero_Citation_Dialog = new function () {
 	 */
 	function cancel() {
 		io.citation.citationItems = new Array();
+		io.accept();
 	}
 	
 	/*
@@ -622,8 +622,24 @@ var Zotero_Citation_Dialog = new function () {
 			_getCitation();
 			
 			editor.readonly = !io.citation.citationItems.length;
-			editor.value = (io.citation.citationItems.length ? io.previewFunction() : "");
-			_originalHTML = editor.value;
+			if(io.citation.citationItems.length) {
+				io.preview(function(preview) {
+					editor.value = preview;
+					
+					if(editor.initialized) {
+						_originalHTML = editor.value;
+					} else {
+						var eventListener = function() {
+							_originalHTML = editor.value;
+							editor.removeEventListener("tinymceInitialized", eventListener, false);
+						};
+						editor.addEventListener("tinymceInitialized", eventListener, false);
+					}
+				});
+			} else {
+				editor.value = "";
+				_originalHTML = "";
+			}
 		}
 	}
 	
