@@ -42,7 +42,7 @@ var Zotero_LocateMenu = new function() {
 			locateMenu.removeChild(locateMenu.firstChild);
 		}
 		
-		var selectedItems = [item for each(item in ZoteroPane_Local.getSelectedItems()) if(!item.isNote())];
+		var selectedItems = _getSelectedItems();
 		
 		if(selectedItems.length) {
 			_addViewOptions(locateMenu, selectedItems, true, true);
@@ -50,7 +50,7 @@ var Zotero_LocateMenu = new function() {
 			var availableEngines = _getAvailableLocateEngines(selectedItems);
 			// add engines that are available for selected items
 			if(availableEngines.length) {
-				_addLocateEngines(locateMenu, availableEngines, true);
+				Zotero_LocateMenu.addLocateEngines(locateMenu, availableEngines, null, true);
 			}
 		} else {
 			// add "no items selected"
@@ -96,10 +96,10 @@ var Zotero_LocateMenu = new function() {
 	 */
 	this.buildContextMenu = function(menu) {
 		// get selected items
-		var selectedItems = [item for each(item in ZoteroPane_Local.getSelectedItems()) if(!item.isNote())];
+		var selectedItems = _getSelectedItems();
 		
-		// if no items selected, stop now
-		if(!selectedItems.length) return;
+		// if no items selected or >20 items selected, stop now
+		if(!selectedItems.length || selectedItems.length > 20) return;
 		
 		// add view options
 		_addViewOptions(menu, selectedItems);
@@ -207,16 +207,21 @@ var Zotero_LocateMenu = new function() {
 	/**
 	 * Add locate engine options to a menu
 	 * @param {menupopup} menu The menu to add menu items to
-	 * @param {Zotero.LocateManater.LocateEngine[]} engines The list of engines to add to the menu
+	 * @param {Zotero.LocateManager.LocateEngine[]} engines The list of engines to add to the menu
+	 * @param {Function|null} items Function to call to locate items
 	 * @param {Boolean} showIcons Whether menu items should have associated icons
 	 */
-	function _addLocateEngines(menu, engines, showIcons) {
+	this.addLocateEngines = function(menu, engines, locateFn, showIcons) {
+		if(!locateFn) {
+			locateFn = this.locateItem;
+		}
+		
 		for each(var engine in engines) {
 			var menuitem = _createMenuItem(engine.name, null, engine.description);
 			menuitem.setAttribute("class", "menuitem-iconic");
 			menuitem.setAttribute("image", engine.icon);
 			menu.appendChild(menuitem);
-			menuitem.addEventListener("command", _locateItem, false);
+			menuitem.addEventListener("command", locateFn, false);
 		}
 	}
 	
@@ -270,8 +275,10 @@ var Zotero_LocateMenu = new function() {
 	/**
 	 * Locate selected items
 	 */
-	function _locateItem(event) {
-		var selectedItems = ZoteroPane_Local.getSelectedItems();
+	this.locateItem = function(event, selectedItems) {
+		if(!selectedItems) {
+			selectedItems = _getSelectedItems();
+		}
 		
 		// find selected engine
 		var selectedEngine = Zotero.LocateManager.getEngineByName(event.target.label);
@@ -311,6 +318,19 @@ var Zotero_LocateMenu = new function() {
 		);
 	}
 	
+	/**
+	 * Get the first 50 selected items
+	 */
+	function _getSelectedItems() {
+		var allSelectedItems = ZoteroPane_Local.getSelectedItems();
+		var selectedItems = [];
+		while(selectedItems.length < 50 && allSelectedItems.length) {
+			var item = allSelectedItems.shift();
+			if(!item.isNote()) selectedItems.push(item);
+		}
+		return selectedItems;
+	}
+	
 	var ViewOptions = {};
 	
 	/**
@@ -325,13 +345,13 @@ var Zotero_LocateMenu = new function() {
 		this.canHandleItem = function(item) !!_getFirstAttachmentWithMIMEType(item, this._mimeTypes);
 		
 		this.handleItems = function(items, event) {
+			var attachments = [];
 			for each(var item in items) {
 				var attachment = _getFirstAttachmentWithMIMEType(item, this._mimeTypes);
-				if(attachment) {
-					ZoteroPane_Local.viewAttachment(attachment.id, event);
-					return;
-				}
+				if(attachment) attachments.push(attachment.id);
 			}
+			
+			ZoteroPane_Local.viewAttachment(attachments, event);
 		}
 		
 		function _getFirstAttachmentWithMIMEType(item, mimeTypes) {
@@ -418,13 +438,13 @@ var Zotero_LocateMenu = new function() {
 		this.canHandleItem = function(item) !!_getFile(item);
 		
 		this.handleItems = function(items, event) {
+			var attachments = [];
 			for each(var item in items) {
 				var attachment = _getFile(item);
-				if(attachment) {
-					ZoteroPane_Local.viewAttachment(attachment.id, event);
-					return;
-				}
+				if(attachment) attachments.push(attachment.id);
 			}
+			
+			ZoteroPane_Local.viewAttachment(attachments, event);
 		}
 		
 		function _getFile(item) {
@@ -456,13 +476,13 @@ var Zotero_LocateMenu = new function() {
 		}
 		
 		this.handleItems = function(items, event) {
+			var attachments = [];
 			for each(var item in items) {
 				var attachment = _getBestNonNativeAttachment(item);
-				if(attachment) {
-					ZoteroPane_Local.viewAttachment(attachment.id, event, false, this.useExternalViewer);
-					return;
-				}
+				if(attachment) attachments.push(attachment.id);
 			}
+			
+			ZoteroPane_Local.viewAttachment(attachments, event, false, this.useExternalViewer);
 		}
 		
 		function _getBestNonNativeAttachment(item) {
@@ -517,7 +537,6 @@ var Zotero_LocateMenu = new function() {
 				var attachment = _getBestFile(item);
 				if(attachment) {
 					ZoteroPane_Local.showAttachmentInFilesystem(attachment.id);
-					return;
 				}
 			}
 		}
