@@ -30,14 +30,60 @@ var _io = {};
 
 
 var Zotero_Preferences = {
-
 	onUnload: function () {
 		Zotero_Preferences.Debug_Output.onUnload();
+	},
+	
+	openHelpLink: function () {
+		var url = "http://www.zotero.org/support/preferences/";
+		var helpTopic = document.getElementsByTagName("prefwindow")[0].currentPane.helpTopic;
+		url += helpTopic;
+		
+		// Non-instantApply prefwindows are usually modal, so we can't open in the topmost window,
+		// since it's probably behind the window
+		var instantApply = Zotero.Prefs.get("browser.preferences.instantApply", true);
+		
+		if (instantApply) {
+			window.opener.ZoteroPane_Local.loadURI(url, { shiftKey: true, metaKey: true });
+		}
+		else {
+			if (Zotero.isStandalone) {
+				var io = Components.classes['@mozilla.org/network/io-service;1']
+							.getService(Components.interfaces.nsIIOService);
+				var uri = io.newURI(url, null, null);
+				var handler = Components.classes['@mozilla.org/uriloader/external-protocol-service;1']
+							.getService(Components.interfaces.nsIExternalProtocolService)
+							.getProtocolHandlerInfo('http');
+				handler.preferredAction = Components.interfaces.nsIHandlerInfo.useSystemDefault;
+				handler.launchWithURI(uri, null);
+			}
+			else {
+				var ww = Components.classes["@mozilla.org/embedcomp/window-watcher;1"]
+							.getService(Components.interfaces.nsIWindowWatcher);
+				var win = ww.openWindow(
+					window,
+					url,
+					"helpWindow",
+					"chrome=no,menubar=yes,location=yes,toolbar=yes,personalbar=yes,resizable=yes,scrollbars=yes,status=yes",
+					null
+				);
+			}
+		}
 	}
 }
 
 function init()
 {
+	if(Zotero.isConnector) {
+		Zotero.activateStandalone();
+		window.close();
+		return;
+	}
+	
+	observerService.addObserver(function() {
+		if(Zotero.isConnector) window.close();
+	}, "zotero-reloaded", false);
+	
 	// Display the appropriate modifier keys for the platform
 	var rows = document.getElementById('zotero-prefpane-keys').getElementsByTagName('row');
 	for (var i=0; i<rows.length; i++) {
@@ -73,6 +119,11 @@ function init()
 	} else if(document.location.hash == "#cite") {
 		document.getElementById('zotero-prefs').showPane(document.getElementById("zotero-prefpane-cite"));
 	}
+	
+	var showInAppTab;
+	if(!Zotero.isFx4 && (showInAppTab = document.getElementById("zotero-prefpane-general-showIn-appTab"))) {
+		showInAppTab.setAttribute("hidden", "true");
+	}
 }
 
 
@@ -537,7 +588,9 @@ function populateQuickCopyList() {
 	menulist.setAttribute('preference', "pref-quickCopy-setting");
 	updateQuickCopyHTMLCheckbox();
 	
-	refreshQuickCopySiteList();
+	if (!Zotero.isStandalone) {
+		refreshQuickCopySiteList();
+	}
 }
 
 
@@ -709,11 +762,18 @@ function deleteSelectedQuickCopySite() {
 
 function updateQuickCopyInstructions() {
 	var prefix = Zotero.isMac ? 'Cmd+Shift+' : 'Ctrl+Alt+';
+	
 	var key = Zotero.Prefs.get('keys.copySelectedItemsToClipboard');
-	
-	var instr = document.getElementById('quickCopy-instructions');
 	var str = Zotero.getString('zotero.preferences.export.quickCopy.instructions', prefix + key);
+	var instr = document.getElementById('quickCopy-instructions');
+	while (instr.hasChildNodes()) {
+		instr.removeChild(instr.firstChild);
+	}
+	instr.appendChild(document.createTextNode(str));
 	
+	var key = Zotero.Prefs.get('keys.copySelectedItemCitationsToClipboard');
+	var str = Zotero.getString('zotero.preferences.export.quickCopy.citationInstructions', prefix + key);
+	var instr = document.getElementById('quickCopy-citationInstructions');
 	while (instr.hasChildNodes()) {
 		instr.removeChild(instr.firstChild);
 	}
@@ -816,7 +876,7 @@ function updatePDFToolsStatus() {
 			else if (part.type == 'link') {
 				var elem = document.createElement('label');
 				elem.setAttribute('value', part.text);
-				elem.setAttribute('class', 'text-link');
+				elem.setAttribute('class', 'zotero-text-link');
 				for (var key in part.attributes) {
 					elem.setAttribute(key, part.attributes[key]);
 					
@@ -846,7 +906,7 @@ function updatePDFToolsStatus() {
 			else if (part.type == 'link') {
 				var elem = document.createElement('label');
 				elem.setAttribute('value', part.text);
-				elem.setAttribute('class', 'text-link');
+				elem.setAttribute('class', 'zotero-text-link');
 				for (var key in part.attributes) {
 					elem.setAttribute(key, part.attributes[key]);
 					
@@ -1174,12 +1234,9 @@ function revealDataDirectory() {
 		dataDir.reveal();
 	}
 	catch (e) {
-		// On platforms that don't support nsILocalFile.reveal() (e.g. Linux), we
-		// open a small window with a selected read-only textbox containing the
-		// file path, so the user can open it, Control-c, Control-w, Alt-Tab, and
-		// Control-v the path into another app
-		var io = {alertText: dataDir.path};
-		window.openDialog('chrome://zotero/content/selectableAlert.xul', "zotero-reveal-window", "chrome", io);
+		// On platforms that don't support nsILocalFile.reveal() (e.g. Linux),
+		// launch the directory
+		window.opener.ZoteroPane_Local.launchFile(dataDir);
 	}
 }
 
@@ -1191,6 +1248,55 @@ function runIntegrityCheck() {
 	var ok = Zotero.DB.integrityCheck();
 	if (ok) {
 		ok = Zotero.Schema.integrityCheck();
+		if (!ok) {
+			var buttonFlags = (ps.BUTTON_POS_0) * (ps.BUTTON_TITLE_IS_STRING)
+				+ (ps.BUTTON_POS_1) * (ps.BUTTON_TITLE_CANCEL);
+			var index = ps.confirmEx(window,
+				Zotero.getString('general.failed'),
+				Zotero.getString('db.integrityCheck.failed') + "\n\n" +
+					Zotero.getString('db.integrityCheck.repairAttempt') + " " +
+					Zotero.getString('db.integrityCheck.appRestartNeeded', Zotero.appName),
+				buttonFlags,
+				Zotero.getString('db.integrityCheck.fixAndRestart', Zotero.appName),
+				null, null, null, {}
+			);
+			
+			if (index == 0) {
+				// Safety first
+				Zotero.DB.backupDatabase();
+				
+				// Fix the errors
+				Zotero.Schema.integrityCheck(true);
+				
+				// And run the check again
+				ok = Zotero.Schema.integrityCheck();
+				var buttonFlags = (ps.BUTTON_POS_0) * (ps.BUTTON_TITLE_IS_STRING);
+				if (ok) {
+					var str = 'success';
+					var msg = Zotero.getString('db.integrityCheck.errorsFixed');
+				}
+				else {
+					var str = 'failed';
+					var msg = Zotero.getString('db.integrityCheck.errorsNotFixed')
+								+ "\n\n" + Zotero.getString('db.integrityCheck.reportInForums');
+				}
+				
+				ps.confirmEx(window,
+					Zotero.getString('general.' + str),
+					msg,
+					buttonFlags,
+					Zotero.getString('general.restartApp', Zotero.appName),
+					null, null, null, {}
+				);
+				
+				var appStartup = Components.classes["@mozilla.org/toolkit/app-startup;1"]
+						.getService(Components.interfaces.nsIAppStartup);
+				appStartup.quit(Components.interfaces.nsIAppStartup.eAttemptQuit
+					| Components.interfaces.nsIAppStartup.eRestart);
+			}
+			
+			return;
+		}
 	}
 	var str = ok ? 'passed' : 'failed';
 	
@@ -1298,9 +1404,13 @@ Zotero_Preferences.Debug_Output = {
 	
 	
 	toggleStore: function () {
-		var storing = Zotero.Debug.storing;
-		Zotero.Debug.setStore(!storing);
-		if (!storing) {
+		this.setStore(!Zotero.Debug.storing);
+	},
+	
+	
+	setStore: function (set) {
+		Zotero.Debug.setStore(set);
+		if (set) {
 			this._initTimer();
 		}
 		else {
@@ -1315,20 +1425,7 @@ Zotero_Preferences.Debug_Output = {
 	
 	
 	view: function () {
-		var uri = "zotero://debug/";
-		var features = "menubar=yes,toolbar=no,location=no,scrollbars,centerscreen,resizable";
-		
-		var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"]
-		.getService(Components.interfaces.nsIWindowMediator);
-		var win = wm.getMostRecentWindow("navigator:browser");
-		if (win) {
-			win.open(uri, null, features);
-		}
-		else {
-			var ww = Components.classes["@mozilla.org/embedcomp/window-watcher;1"]
-						.getService(Components.interfaces.nsIWindowWatcher);
-			var win = ww.openWindow(null, uri, null, features + ",width=775,height=575", null);
-		}
+		openInViewer("zotero://debug/");
 	},
 	
 	
@@ -1342,6 +1439,7 @@ Zotero_Preferences.Debug_Output = {
 			Zotero.Prefs.get('debug.store.submitSize'),
 			Zotero.Prefs.get('debug.store.submitLineLength')
 		);
+		Zotero_Preferences.Debug_Output.setStore(false);
 		
 		var uploadCallback = function (xmlhttp) {
 			document.getElementById('debug-output-submit').disabled = false;
@@ -1373,7 +1471,7 @@ Zotero_Preferences.Debug_Output = {
 			var reportID = reported[0].getAttribute('reportID');
 			ps.alert(
 				null,
-				"Submitted",
+				"Debug Output Submitted",
 				"Debug output has been sent to the Zotero server.\n\n"
 					+ "The Debug ID is D" + reportID + "."
 			);
@@ -1778,6 +1876,7 @@ function updateProxyPrefs() {
 		document.getElementById('zotero-proxies-autoRecognize').disabled = 
 		document.getElementById('zotero-proxies-disableByDomain-checkbox').disabled = 
 		document.getElementById('zotero-proxies-disableByDomain-textbox').disabled = !transparent;
+
 }
 
 /**
@@ -1797,8 +1896,9 @@ function updateWordProcessorInstructions() {
  */
 function handleShowInPreferenceChange() {
 	var showInSeparateTab = document.getElementById("zotero-prefpane-general-showIn-separateTab");
+	var showInAppTab = document.getElementById("zotero-prefpane-general-showIn-appTab");
 	if(Zotero.isFx4) {
-		if(showInSeparateTab.selected) {
+		if(showInAppTab.selected) {
 			document.getElementById('statusBarIcon').selectedItem = document.getElementById('statusBarIcon-none');
 			Zotero.Prefs.set("statusBarIcon", 0);
 		} else if(Zotero.isFx4) {
@@ -1806,4 +1906,37 @@ function handleShowInPreferenceChange() {
 			Zotero.Prefs.set("statusBarIcon", 2);
 		}
 	}
+}
+
+/**
+ * Opens a URI in the basic viewer in Standalone, or a new window in Firefox
+ */
+function openInViewer(uri, newTab) {
+	var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"]
+		.getService(Components.interfaces.nsIWindowMediator);
+	const features = "menubar=yes,toolbar=no,location=no,scrollbars,centerscreen,resizable";
+	
+	if(Zotero.isStandalone) {
+		var win = wm.getMostRecentWindow("zotero:basicViewer");
+		if(win) {
+			win.loadURI(uri);
+		} else {
+			window.openDialog("chrome://zotero/content/standalone/basicViewer.xul",
+				"basicViewer", "chrome,resizable,centerscreen", uri);
+		}
+	} else {
+		var win = wm.getMostRecentWindow("navigator:browser");
+		if(win) {
+			if(newTab) {
+				win.gBrowser.selectedTab = win.gBrowser.addTab(uri);
+			} else {
+				win.open(uri, null, features);
+			}
+		}
+		else {
+			var ww = Components.classes["@mozilla.org/embedcomp/window-watcher;1"]
+						.getService(Components.interfaces.nsIWindowWatcher);
+			var win = ww.openWindow(null, uri, null, features + ",width=775,height=575", null);
+		}
+	}
 }
\ No newline at end of file
