@@ -102,6 +102,10 @@ Zotero.Attachments = new function(){
 			// hmph
 			Zotero.DB.rollbackTransaction();
 			
+			var msg = "Failed importing file " + file.path;
+			Components.utils.reportError(msg);
+			Zotero.debug(msg, 1);
+			
 			try {
 				// Clean up
 				if (itemID) {
@@ -199,7 +203,8 @@ Zotero.Attachments = new function(){
 	}
 	
 	
-	function importFromURL(url, sourceItemID, forceTitle, forceFileBaseName, parentCollectionIDs, mimeType, libraryID) {
+	function importFromURL(url, sourceItemID, forceTitle, forceFileBaseName, parentCollectionIDs,
+			mimeType, libraryID, callback, cookieSandbox) {
 		Zotero.debug('Importing attachment from URL');
 		
 		if (sourceItemID && parentCollectionIDs) {
@@ -220,28 +225,14 @@ Zotero.Attachments = new function(){
 		
 		// Save using a hidden browser
 		var nativeHandlerImport = function () {
-			var browser = Zotero.Browser.createHiddenBrowser();
-			var imported = false;
-			var onpageshow = function() {
-				// ignore spurious about:blank loads
-				if(browser.contentDocument.location.href == "about:blank") return;
-				
-				// pageshow can be triggered multiple times on some pages,
-				// so make sure we only import once
-				// (https://www.zotero.org/trac/ticket/795)
-				if (imported) {
-					return;
-				}
-				var callback = function () {
-					browser.removeEventListener("pageshow", onpageshow, false);
+			var browser = Zotero.HTTP.processDocuments(url, function() {
+				var importCallback = function (item) {
 					Zotero.Browser.deleteHiddenBrowser(browser);
+					if(callback) callback(item);
 				};
 				Zotero.Attachments.importFromDocument(browser.contentDocument,
-					sourceItemID, forceTitle, parentCollectionIDs, callback, libraryID);
-				imported = true;
-			};
-			browser.addEventListener("pageshow", onpageshow, false);
-			browser.loadURI(url);
+					sourceItemID, forceTitle, parentCollectionIDs, importCallback, libraryID);
+			}, undefined, undefined, true);
 		};
 		
 		// Save using remote web browser persist
@@ -261,6 +252,7 @@ Zotero.Attachments = new function(){
 				.classes["@mozilla.org/embedding/browser/nsWebBrowserPersist;1"]
 				.createInstance(nsIWBP);
 			wbp.persistFlags = nsIWBP.PERSIST_FLAGS_AUTODETECT_APPLY_CONVERSION;
+			if(cookieSandbox) cookieSandbox.attachToInterfaceRequestor(wbp);
 			var encodingFlags = false;
 			
 			Zotero.DB.beginTransaction();
@@ -326,11 +318,9 @@ Zotero.Attachments = new function(){
 						// if it fails
 						//
 						// TODO: index later
-						var timer = Components.classes["@mozilla.org/timer;1"].
-							createInstance(Components.interfaces.nsITimer);
-						timer.initWithCallback({notify: function() {
+						setTimeout(function() {
 							Zotero.Fulltext.indexItems([itemID]);
-						}}, 1000, Components.interfaces.nsITimer.TYPE_ONE_SHOT);
+						}, 1000);
 					}
 					catch (e) {
 						// Clean up
@@ -356,6 +346,8 @@ Zotero.Attachments = new function(){
 				nsIURL.spec = url;
 				wbp.saveURI(nsIURL, null, null, null, null, file);
 				
+				if(callback) callback(attachmentItem);
+				
 				return attachmentItem;
 			}
 			catch (e){
@@ -389,12 +381,12 @@ Zotero.Attachments = new function(){
 		}
 		
 		if (mimeType) {
-			return process(mimeType);
+			return process(mimeType, Zotero.MIME.hasNativeHandler(mimeType));
 		}
 		else {
 			Zotero.MIME.getMIMETypeFromURL(url, function (mimeType, hasNativeHandler) {
 				process(mimeType, hasNativeHandler);
-			});
+			}, cookieSandbox);
 		}
 	}
 	
@@ -500,6 +492,10 @@ Zotero.Attachments = new function(){
 		var url = document.location.href;
 		var title = forceTitle ? forceTitle : document.title;
 		var mimeType = document.contentType;
+		if(Zotero.Attachments.isPDFJS(document)) {
+			mimeType = "application/pdf";
+		}
+		
 		var charsetID = Zotero.CharacterSets.getID(document.characterSet);
 		
 		if (!forceTitle) {
@@ -566,16 +562,16 @@ Zotero.Attachments = new function(){
 				var sync = true;
 				
 				// Load WebPageDump code
+				var wpd = {"Zotero":Zotero};
 				Components.classes["@mozilla.org/moz/jssubscript-loader;1"]
 					.getService(Components.interfaces.mozIJSSubScriptLoader)
-					.loadSubScript("chrome://zotero/content/webpagedump/common.js");
-				
+					.loadSubScript("chrome://zotero/content/webpagedump/common.js", wpd);
 				Components.classes["@mozilla.org/moz/jssubscript-loader;1"]
 					.getService(Components.interfaces.mozIJSSubScriptLoader)
-					.loadSubScript("chrome://zotero/content/webpagedump/domsaver.js");
+					.loadSubScript("chrome://zotero/content/webpagedump/domsaver.js", wpd);
 				
-				wpdDOMSaver.init(file.path, document);
-				wpdDOMSaver.saveHTMLDocument();
+				wpd.wpdDOMSaver.init(file.path, document);
+				wpd.wpdDOMSaver.saveHTMLDocument();
 				
 				attachmentItem.attachmentPath = this.getPath(
 					file, Zotero.Attachments.LINK_MODE_IMPORTED_URL
@@ -608,10 +604,9 @@ Zotero.Attachments = new function(){
 						// we'll index it later if it fails
 						//
 						// TODO: index later
-						var timer = Components.classes["@mozilla.org/timer;1"].
-							createInstance(Components.interfaces.nsITimer);
-						timer.initWithCallback({notify: f}, 1000,
-							Components.interfaces.nsITimer.TYPE_ONE_SHOT);
+						setTimeout(function () {
+							f();
+						}, 1000);
 					}
 					catch (e) {
 						// Clean up
@@ -648,10 +643,9 @@ Zotero.Attachments = new function(){
 				Zotero.Notifier.trigger('add', 'item', itemID);
 				
 				// Wait a second before indexing (see note above)
-				var timer = Components.classes["@mozilla.org/timer;1"].
-					createInstance(Components.interfaces.nsITimer);
-				timer.initWithCallback({notify: f}, 1000,
-					Components.interfaces.nsITimer.TYPE_ONE_SHOT);
+				setTimeout(function () {
+					f();
+				}, 1000);
 			}
 			
 			// Caution: Take care using this itemID. The notifier may not yet have been called,
@@ -1356,4 +1350,22 @@ Zotero.Attachments = new function(){
 					.getURLSpecFromFile(file);
 		browser.loadURI(url);
 	}
+	
+	/**
+	 * Determines if a given document is an instance of PDFJS
+	 * @return {Boolean}
+	 */
+	this.isPDFJS = function(doc) {
+		// pdf.js HACK
+		if(doc.contentType === "text/html") {
+			var win = doc.defaultView;
+			if(win) {
+				win = win.wrappedJSObject;
+				if(win && "PDFJS" in win && win.PDFJS.isFirefoxExtension) {
+					return true;
+				}
+			}
+		}
+		return false;
+	}
 }
