@@ -49,15 +49,21 @@ Zotero.Items = new function() {
 				['firstCreator', 'numNotes', 'numAttachments']
 			);
 		}
+		
+		// Once primary fields have been cached, get rid of getter for speed purposes
+		delete this.primaryFields;
+		this.primaryFields = _primaryFields;
+		
 		return _primaryFields;
 	});
 	
-	this.__defineGetter__('linkedItemPredicate', function () "owl:sameAs");
 	
 	// Private members
 	var _cachedFields = [];
 	var _firstCreatorSQL = '';
 	var _primaryFields = [];
+	var _emptyTrashIdleObserver = null;
+	var _emptyTrashTimer = null;
 	
 	
 	/*
@@ -126,16 +132,34 @@ Zotero.Items = new function() {
 	/**
 	 * Return items marked as deleted
 	 *
-	 * @param	{Boolean}	asIDs			Return itemIDs instead of
+	 * @param {Number|NULL}  libraryID
+	 * @param {Boolean}  asIDs			Return itemIDs instead of
 	 *											Zotero.Item objects
-	 * @return	{Zotero.Item[]|Integer[]}
+	 * @return {Zotero.Item[]|Integer[]}
 	 */
-	this.getDeleted = function (asIDs, days) {
-		var sql = "SELECT itemID FROM deletedItems";
+	this.getDeleted = function (libraryID, asIDs, days) {
+		// Throw warning for pre-3.0b3 arguments
+		if (typeof libraryID == 'boolean') {
+			throw new Error("libraryID must be a number or null");
+		}
+		
+		var sql = "SELECT itemID FROM items JOIN deletedItems USING (itemID) "
+				+ "WHERE libraryID"  + (libraryID ? "=?" : " IS NULL");
+		
 		if (days) {
-			sql += " WHERE dateDeleted<=DATE('NOW', '-" + parseInt(days) + " DAYS')";
+			sql += " AND dateDeleted<=DATE('NOW', '-" + parseInt(days) + " DAYS')";
+		}
+		
+		if (libraryID) {
+			var ids = Zotero.DB.columnQuery(sql, [libraryID]);
+		}
+		else {
+			var ids = Zotero.DB.columnQuery(sql);
+		}
+		
+		if (!ids) {
+			return [];
 		}
-		var ids = Zotero.DB.columnQuery(sql);
 		if (asIDs) {
 			return ids;
 		}
@@ -374,6 +398,69 @@ Zotero.Items = new function() {
 	}
 	
 	
+	this.merge = function (item, otherItems) {
+		Zotero.DB.beginTransaction();
+		
+		var otherItemIDs = [];  
+		var itemURI = Zotero.URI.getItemURI(item);
+		
+		for each(var otherItem in otherItems) {
+			// Move child items to master
+			var ids = otherItem.getAttachments(true).concat(otherItem.getNotes(true));
+			for each(var id in ids) {
+				var attachment = Zotero.Items.get(id);
+				
+				// TODO: Skip identical children?
+				
+				attachment.setSource(item.id);
+				attachment.save();
+			}
+			
+			// All other operations are additive only and do not affect the,
+			// old item, which will be put in the trash
+			
+			// Add collections to master
+			var collectionIDs = otherItem.getCollections();
+			for each(var collectionID in collectionIDs) {
+				var collection = Zotero.Collections.get(collectionID);
+				collection.addItem(item.id);
+			}
+			
+			// Add tags to master
+			var tags = otherItem.getTags();
+			for each(var tag in tags) {
+				item.addTagByID(tag.id);
+			}
+			
+			// Related items
+			var relatedItems = otherItem.relatedItemsBidirectional;
+			Zotero.debug(item._getRelatedItems(true));
+			for each(var relatedItemID in relatedItems) {
+				item.addRelatedItem(relatedItemID);
+			}
+			
+			// Relations
+			Zotero.Relations.copyURIs(
+				item.libraryID,
+				Zotero.URI.getItemURI(otherItem),
+				Zotero.URI.getItemURI(item)
+			);
+			
+			// Add relation to track merge
+			var otherItemURI = Zotero.URI.getItemURI(otherItem);
+			Zotero.Relations.add(item.libraryID, otherItemURI, Zotero.Relations.deletedItemPredicate, itemURI);
+			
+			// Trash other item
+			otherItem.deleted = true;
+			otherItem.save();
+		}
+		
+		item.save();
+		
+		Zotero.DB.commitTransaction();
+	}
+	
+	
 	this.trash = function (ids) {
 		ids = Zotero.flattenArguments(ids);
 		
@@ -405,15 +492,77 @@ Zotero.Items = new function() {
 	/**
 	 * @param	{Integer}	days	Only delete items deleted more than this many days ago
 	 */
-	this.emptyTrash = function (days) {
+	this.emptyTrash = function (libraryID, days, limit) {
+		var t = new Date();
+		
 		Zotero.DB.beginTransaction();
-		var deletedIDs = this.getDeleted(true, days);
-		if (deletedIDs) {
+		var deletedIDs = this.getDeleted(libraryID, true, days);
+		if (deletedIDs.length) {
+			if (limit) {
+				deletedIDs = deletedIDs.slice(0, limit - 1)
+			}
 			this.erase(deletedIDs);
-			Zotero.Notifier.trigger('refresh', 'collection', 0);
+			Zotero.Notifier.trigger('refresh', 'trash', libraryID ? libraryID : 0);
 		}
 		Zotero.DB.commitTransaction();
-		return deletedIDs ? deletedIDs.length : 0;
+		
+		if (deletedIDs.length) {
+			Zotero.debug("Emptied " + deletedIDs.length + " item(s) from trash in " + (new Date() - t) + " ms");
+		}
+		
+		return deletedIDs.length;
+	}
+	
+	
+	/**
+	 * Start idle observer to delete trashed items older than a certain number of days
+	 */
+	this.startEmptyTrashTimer = function () {
+		_emptyTrashIdleObserver = {
+			observe: function (subject, topic, data) {
+				if (topic == 'idle' || topic == 'timer-callback') {
+					var days = Zotero.Prefs.get('trashAutoEmptyDays');
+					if (!days) {
+						return;
+					}
+					
+					// TODO: empty group trashes if permissions
+					
+					// Delete a few items a time
+					//
+					// TODO: increase number after dealing with slow
+					// tag.getLinkedItems() call during deletes
+					var num = 10;
+					var deleted = Zotero.Items.emptyTrash(null, days, num);
+					
+					if (!deleted) {
+						_emptyTrashTimer = null;
+						return;
+					}
+					
+					// Set a timer to do more every few seconds
+					if (!_emptyTrashTimer) {
+						_emptyTrashTimer = Components.classes["@mozilla.org/timer;1"].
+											createInstance(Components.interfaces.nsITimer);
+					}
+					_emptyTrashTimer.init(
+						_emptyTrashIdleObserver.observe,
+						5 * 1000,
+						Components.interfaces.nsITimer.TYPE_ONE_SHOT
+					);
+				}
+				// When no longer idle, cancel timer
+				else if (topic == 'back') {
+					if (_emptyTrashTimer) {
+						_emptyTrashTimer.cancel();
+					}
+				}
+			}
+		};
+		
+		var idleService = Components.classes["@mozilla.org/widget/idleservice;1"].
+							getService(Components.interfaces.nsIIdleService);
+		idleService.addIdleObserver(_emptyTrashIdleObserver, 305);
 	}
 	
 	
@@ -605,28 +754,29 @@ Zotero.Items = new function() {
 		if (arguments[0]) {
 			sql += ' AND I.itemID IN (' + Zotero.join(arguments[0], ',') + ')';
 		}
-		var itemsRows = Zotero.DB.query(sql);
-		var itemIDs = [];
-		for each(var row in itemsRows) {
-			var itemID = row.itemID;
-			itemIDs.push(itemID);
+		var itemsRows = Zotero.DB.query(sql),
+			itemIDs = {};
+		for(var i=0, n=itemsRows.length; i<n; i++) {
+			var row = itemsRows[i],
+				itemID = row.itemID;
+			itemIDs[itemID] = true;
 			
 			// Item doesn't exist -- create new object and stuff in array
-			if (!this._objectCache[row.itemID]) {
+			if (!this._objectCache[itemID]) {
 				var item = new Zotero.Item();
 				item.loadFromRow(row, true);
-				this._objectCache[row.itemID] = item;
+				this._objectCache[itemID] = item;
 			}
 			// Existing item -- reload in place
 			else {
-				this._objectCache[row.itemID].loadFromRow(row, true);
+				this._objectCache[itemID].loadFromRow(row, true);
 			}
 		}
 		
 		// If loading all items, remove old items that no longer exist
 		if (!arguments[0]) {
 			for each(var c in this._objectCache) {
-				if (itemIDs.indexOf(c.id) == -1) {
+				if (!itemIDs[c.id]) {
 					this.unload(c.id);
 				}
 			}
