@@ -337,23 +337,73 @@ Zotero.Tag.prototype.save = function (full) {
 			key
 		];
 		
-		if (isNew) {
-			var sql = "INSERT INTO tags (" + columns.join(', ') + ") VALUES ("
-				+ placeholders.join(', ') + ")";
-			var insertID = Zotero.DB.query(sql, sqlValues);
-			if (!tagID) {
-				tagID = insertID;
+		try {
+			if (isNew) {
+				var sql = "INSERT INTO tags (" + columns.join(', ') + ") VALUES ("
+					+ placeholders.join(', ') + ")";
+				var insertID = Zotero.DB.query(sql, sqlValues);
+				if (!tagID) {
+					tagID = insertID;
+				}
+			}
+			else {
+				// Remove tagID from beginning
+				columns.shift();
+				sqlValues.shift();
+				sqlValues.push(tagID);
+				
+				var sql = "UPDATE tags SET " + columns.join("=?, ") + "=?"
+					+ " WHERE tagID=?";
+				Zotero.DB.query(sql, sqlValues);
 			}
 		}
-		else {
-			// Remove tagID from beginning
-			columns.shift();
-			sqlValues.shift();
-			sqlValues.push(tagID);
-			
-			var sql = "UPDATE tags SET " + columns.join("=?, ") + "=?"
-				+ " WHERE tagID=?";
-			Zotero.DB.query(sql, sqlValues);
+		catch (e) {
+			// If an incoming tag is the same as an existing tag, but with a different key,
+			// then delete the old tag and add its linked items to the new tag
+			if (typeof e == 'string' && e.indexOf('columns libraryID, name, type are not unique') != -1) {
+				Zotero.debug("Tag matches existing tag with different key -- delete old tag and merging items");
+				
+				// GET existing tag
+				var existing = Zotero.Tags.getIDs(this.name, this.libraryID);
+				if (!existing) {
+					throw new Error("Existing tag not found");
+				}
+				for each(var id in existing) {
+					var tag = Zotero.Tags.get(id, true);
+					if (tag.type == this.type) {
+						var linked = tag.getLinkedItems(true);
+						Zotero.Tags.erase(id);
+						Zotero.Tags.purge(id);
+						break;
+					}
+				}
+				
+				// Save again
+				if (isNew) {
+					var sql = "INSERT INTO tags (" + columns.join(', ') + ") VALUES ("
+						+ placeholders.join(', ') + ")";
+					var insertID = Zotero.DB.query(sql, sqlValues);
+					if (!tagID) {
+						tagID = insertID;
+					}
+				}
+				else {
+					var sql = "UPDATE tags SET " + columns.join("=?, ") + "=?"
+						+ " WHERE tagID=?";
+					Zotero.DB.query(sql, sqlValues);
+				}
+				
+				// TEMP: until getLinkedItems() returns only arrays
+				linked = linked ? linked : [];
+				var linked2 = this.getLinkedItems(true);
+				linked2 = linked2 ? linked2 : [];
+				linked = linked.concat(linked2);
+				
+				this.linkedItems = Zotero.Utilities.arrayUnique(linked);
+			}
+			else {
+				throw (e);
+			}
 		}
 		
 		
@@ -397,6 +447,12 @@ Zotero.Tag.prototype.save = function (full) {
 				var done = 0;
 				var maxItems = 998; // stay below compiled limit
 				var numItems = removed.length;
+				
+				var pairs = [];
+				for each(var itemID in removed) {
+					pairs.push(itemID + "-" + tagID);
+				}
+				
 				var tempRemoved = removed;
 				
 				do {
@@ -409,12 +465,15 @@ Zotero.Tag.prototype.save = function (full) {
 					done += chunk.length;
 				}
 				while (done < numItems);
+				
+				Zotero.Notifier.trigger('remove', 'item-tag', pairs);
 			}
 			
 			if (newids.length) {
 				var sql = "INSERT INTO itemTags (itemID, tagID) VALUES (?,?)";
 				var insertStatement = Zotero.DB.getStatement(sql);
 				
+				var pairs = [];
 				for each(var itemID in newids) {
 					insertStatement.bindInt32Parameter(0, itemID);
 					insertStatement.bindInt32Parameter(1, tagID);
@@ -427,11 +486,13 @@ Zotero.Tag.prototype.save = function (full) {
 						Zotero.debug("tagID: " + tagID);
 						throw (e + ' [ERROR: ' + Zotero.DB.getLastErrorString() + ']');
 					}
+					
+					pairs.push(itemID + "-" + tagID);
 				}
+				
+				Zotero.Notifier.trigger('add', 'item-tag', pairs);
 			}
 			
-			//Zotero.Notifier.trigger('add', 'tag-item', this.id + '-' + itemID);
-			
 			// TODO: notify linked items of name changes?
 			Zotero.Notifier.trigger('modify', 'item', removed.concat(newids));
 		}
