@@ -50,30 +50,35 @@ Zotero.Date = new function(){
 	/**
 	 * Load dateFormat bundle into _dateFormatsBundle
 	 */
-	function _loadDateFormatsBundle() {
-		var src = 'chrome://global/locale/dateFormat.properties';
-		var localeService = Components.classes['@mozilla.org/intl/nslocaleservice;1'].
-							getService(Components.interfaces.nsILocaleService);
-		var appLocale = localeService.getApplicationLocale();
+	this.getMonths = function() {
+		if(_months) return _months;
 		
-		var bundle =
-			Components.classes["@mozilla.org/intl/stringbundle;1"]
-			.getService(Components.interfaces.nsIStringBundleService).createBundle(src, appLocale);
-		
-		_months = {"short":[], "long":[]};
-		for(let i=1; i<=12; i++) {
-			_months.short.push(bundle.GetStringFromName("month."+i+".Mmm"));
-			_months.long.push(bundle.GetStringFromName("month."+i+".name"));
+		if(Zotero.isFx && !Zotero.isBookmarklet) {
+			var src = 'chrome://global/locale/dateFormat.properties';
+			var localeService = Components.classes['@mozilla.org/intl/nslocaleservice;1'].
+								getService(Components.interfaces.nsILocaleService);
+			var appLocale = localeService.getApplicationLocale();
+			
+			var bundle =
+				Components.classes["@mozilla.org/intl/stringbundle;1"]
+				.getService(Components.interfaces.nsIStringBundleService).createBundle(src, appLocale);
+			
+			_months = {"short":[], "long":[]};
+			for(var i=1; i<=12; i++) {
+				_months.short.push(bundle.GetStringFromName("month."+i+".Mmm"));
+				_months.long.push(bundle.GetStringFromName("month."+i+".name"));
+			}
+		} else {
+			// TODO localize for Chrome/Safari
+			_months = {
+				"short":["Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep",
+					"Oct", "Nov", "Dec"],
+				"long":["January", "February", "March", "April", "May", "June", "July",
+					"Auguest", "September", "October", "November", "December"]};
 		}
-	}
-	
-	/**
-	 * Lazy getter for reading month strings from dateFormat.properties
-	 */
-	this.__defineGetter__("months", function() {
-		if(!_months) _loadDateFormatsBundle();
+		
 		return _months;
-	});
+	}
 	
 	/**
 	* Convert an SQL date in the form '2006-06-13 11:03:05' into a JS Date object
@@ -225,13 +230,13 @@ Zotero.Date = new function(){
 	function strToDate(string) {
 		// Parse 'yesterday'/'today'/'tomorrow'
 		var lc = (string + '').toLowerCase();
-		if (lc == 'yesterday' || lc == Zotero.getString('date.yesterday')) {
+		if (lc == 'yesterday' || (Zotero.getString && lc === Zotero.getString('date.yesterday'))) {
 			string = Zotero.Date.dateToSQL(new Date(new Date().getTime() - 86400000)).substr(0, 10);
 		}
-		else if (lc == 'today' || lc == Zotero.getString('date.today')) {
+		else if (lc == 'today' || (Zotero.getString && lc == Zotero.getString('date.today'))) {
 			string = Zotero.Date.dateToSQL(new Date()).substr(0, 10);
 		}
-		else if (lc == 'tomorrow' || lc == Zotero.getString('date.tomorrow')) {
+		else if (lc == 'tomorrow' || (Zotero.getString && lc == Zotero.getString('date.tomorrow'))) {
 			string = Zotero.Date.dateToSQL(new Date(new Date().getTime() + 86400000)).substr(0, 10);
 		}
 		
@@ -306,7 +311,7 @@ Zotero.Date = new function(){
 				}
 				
 				if(date.month) date.month--;		// subtract one for JS style
-				Zotero.debug("DATE: retrieved with algorithms: "+date.toSource());
+				Zotero.debug("DATE: retrieved with algorithms: "+JSON.stringify(date));
 				
 				date.part = m[1]+m[7];
 			} else {
@@ -337,8 +342,9 @@ Zotero.Date = new function(){
 				'aug', 'sep', 'oct', 'nov', 'dec'];
 			// If using a non-English bibliography locale, try those too
 			if (Zotero.locale != 'en-US') {
-				months = months.concat(Zotero.Date.months.short).concat(Zotero.Date.months.long);
-				for(var i in months) months[i] = months[i].toLowerCase();
+				Zotero.Date.getMonths();
+				months = months.concat(_months['short']).concat(_months['long']);
+				for(var i=0, n=months.length; i<n; i++) months[i] = months[i].toLowerCase();
 			}
 			
 			if(!_monthRe) {
@@ -358,7 +364,7 @@ Zotero.Date = new function(){
 		if(!date.day) {
 			// compile day regular expression
 			if(!_dayRe) {
-				var daySuffixes = Zotero.getString("date.daySuffixes").replace(/, ?/g, "|");
+				var daySuffixes = Zotero.getString ? Zotero.getString("date.daySuffixes").replace(/, ?/g, "|") : "";
 				_dayRe = new RegExp("\\b([0-9]{1,2})(?:"+daySuffixes+")?\\b(.*)", "i");
 			}
 			
@@ -416,7 +422,7 @@ Zotero.Date = new function(){
 				string += date.part+" ";
 			}
 			
-			var months = Zotero.Date.months.long;
+			var months = Zotero.Date.getMonths().long;
 			if(date.month != undefined && months[date.month]) {
 				// get short month strings from CSL interpreter
 				string += months[date.month];
@@ -656,73 +662,47 @@ Zotero.Date = new function(){
 	}
 	
 	/**
-	 * Figure out the date order from the output of toLocaleDateString()
+	 * Get the order of the date components based on the current locale
 	 *
 	 * Returns a string with y, m, and d (e.g. 'ymd', 'mdy')
 	 */
 	function getLocaleDateOrder(){
-		if (_localeDateOrder) {
-			return _localeDateOrder;
+		if (!_localeDateOrder) {
+			switch (Zotero.locale.substr(-2)) {
+				// middle-endian
+				case 'US': // The United States
+				case 'BZ': // Belize
+				case 'FM': // The Federated States of Micronesia
+				case 'PA': // Panama
+				case 'PH':	// The Philippines
+				case 'PW':	// Palau
+				case 'ZW': // Zimbabwe
+					_localeDateOrder = 'mdy';
+					break;
+				
+				// big-endian
+				case 'fa': // Persian
+				case 'AL': // Albania
+				case 'CA': // Canada
+				case 'CN': // China
+				case 'HU': // Hungary
+				case 'JP': // Japan
+				case 'KE': // Kenya
+				case 'KR': // Korea
+				case 'LT': // Lithuania
+				case 'LV': // Latvia
+				case 'MN': // Mongolia
+				case 'SE': // Sweden
+				case 'TW': // Taiwan
+				case 'ZA': // South Africa
+					_localeDateOrder = 'ymd';
+					break;
+				
+				// little-endian
+				default:
+					_localeDateOrder = 'dmy';
+			}
 		}
-		
-		var date = new Date("October 5, 2006");
-		var parts = date.toLocaleDateString().match(/([0-9]+)[^0-9]+([0-9]+)[^0-9]+([0-9]+)/);
-		
-		// The above only works on OS X and Linux,
-		// where toLocaleDateString() produces "10/05/2006"
-		if (!parts) {
-				var country = Zotero.locale.substr(3);
-				switch (country) {
-					// I don't know where this country list came from, but these
-					// are little-endian in Zotero.strToDate()
-					case 'US': // The United States
-					case 'FM': // The Federated States of Micronesia
-					case 'PW':	// Palau
-					case 'PH':	// The Philippines
-						return 'mdy';
-						break;
-						
-					default:
-						return 'dmy';
-				}
-		}
-		
-		switch (parseInt(parts[1])){
-			case 2006:
-				var order = 'y';
-				break;
-			case 10:
-				var order = 'm';
-				break;
-			case 5:
-				var order = 'd';
-				break;
-		}
-		switch (parseInt(parts[2])){
-			case 2006:
-				order += 'y';
-				break;
-			case 10:
-				order += 'm';
-				break;
-			case 5:
-				order += 'd';
-				break;
-		}
-		switch (parseInt(parts[3])){
-			case 2006:
-				order += 'y';
-				break;
-			case 10:
-				order += 'm';
-				break;
-			case 5:
-				order += 'd';
-				break;
-		}
-		
-		_localeDateOrder = order;
-		
-		return order;
+		return _localeDateOrder;
 	}
 }
\ No newline at end of file
