@@ -23,6 +23,11 @@
     ***** END LICENSE BLOCK *****
 */
 
+// Exclusive locking mode (default) prevents access to Zotero database while Firefox is open.
+// Normal mode is more convenient for development, but risks database corruption, particularly if
+// the same database is accessed simultaneously by multiple Zotero instances.
+const DB_LOCK_EXCLUSIVE = true;
+
 Zotero.DBConnection = function(dbName) {
 	if (!dbName) {
 		throw ('DB name not provided in Zotero.DBConnection()');
@@ -65,6 +70,7 @@ Zotero.DBConnection = function(dbName) {
 	this._lastTransactionDate = null;
 	this._transactionRollback = null;
 	this._transactionNestingLevel = 0;
+	this._transactionWaitLevel = 0;
 	this._callbacks = { begin: [], commit: [], rollback: [] };
 	this._dbIsCorrupt = null
 	this._self = this;
@@ -129,7 +135,7 @@ Zotero.DBConnection.prototype.query = function (sql,params) {
 				}
 				dataset.push(row);
 			}
-			statement.reset();
+			statement.finalize();
 			
 			return dataset.length ? dataset : false;
 		}
@@ -170,12 +176,12 @@ Zotero.DBConnection.prototype.valueQuery = function (sql,params) {
 	
 	// No rows
 	if (!statement.executeStep()) {
-		statement.reset();
+		statement.finalize();
 		return false;
 	}
 	
 	var value = this._getTypedValue(statement, 0);
-	statement.reset();
+	statement.finalize();
 	return value;
 }
 
@@ -202,7 +208,7 @@ Zotero.DBConnection.prototype.columnQuery = function (sql,params) {
 		while (statement.executeStep()) {
 			column.push(this._getTypedValue(statement, 0));
 		}
-		statement.reset();
+		statement.finalize();
 		return column.length ? column : false;
 	}
 	return false;
@@ -220,13 +226,13 @@ Zotero.DBConnection.prototype.columnQuery = function (sql,params) {
  *		[1,"hello",3] or [{'int':2},{'string':'foobar'}]
  */
 Zotero.DBConnection.prototype.getStatement = function (sql, params, checkParams) {
-	// TODO: limit to Zotero.DB, not all Zotero.DBConnections?
-	if (Zotero.waiting) {
-		throw ("Cannot access database layer during active Zotero.wait()");
-	}
-	
 	var db = this._getDBConnection();
 	
+	// TODO: limit to Zotero.DB, not all Zotero.DBConnections?
+	if (db.transactionInProgress && Zotero.waiting > this._transactionWaitLevel) {
+		throw ("Cannot access database layer from a higher wait level if a transaction is open");
+	}
+	
 	// First, determine the type of query using first word
 	var matches = sql.match(/^[^\s\(]*/);
 	var queryMethod = matches[0].toLowerCase();
@@ -302,11 +308,11 @@ Zotero.DBConnection.prototype.getStatement = function (sql, params, checkParams)
 	if (params) {
 		if (checkParams) {
 			if (numParams == 0) {
-				throw ("Parameters provided for query without placeholders");
+				throw ("Parameters provided for query without placeholders [QUERY: " + sql + "]");
 			}
 			else if (numParams != params.length) {
 				throw ("Incorrect number of parameters provided for query "
-					+ "(" + params.length + ", expecting " + numParams + ")");
+					+ "(" + params.length + ", expecting " + numParams + ") [QUERY: " + sql + "]");
 			}
 		}
 		
@@ -345,7 +351,7 @@ Zotero.DBConnection.prototype.getStatement = function (sql, params, checkParams)
 						}
 						else {
 							var msg = 'Invalid bound parameter ' + params[i]
-								+ ' in ' + Zotero.varDump(params)
+								+ ' in ' + Zotero.Utilities.varDump(params)
 								+ ' [QUERY: ' + sql + ']';
 							Zotero.debug(msg);
 							throw(msg);
@@ -422,21 +428,23 @@ Zotero.DBConnection.prototype.getLastErrorString = function () {
 
 
 Zotero.DBConnection.prototype.beginTransaction = function () {
-	// TODO: limit to Zotero.DB, not all Zotero.DBConnections?
-	if (Zotero.waiting) {
-		var msg = "Cannot access database layer during active Zotero.wait()";
-		Zotero.debug(msg, 2);
-		throw (msg);
-	}
-	
 	var db = this._getDBConnection();
 	
 	if (db.transactionInProgress) {
+		// TODO: limit to Zotero.DB, not all Zotero.DBConnections?
+		if (Zotero.waiting != this._transactionWaitLevel) {
+			var msg = "Cannot start a DB transaction from a different wait level";
+			Zotero.debug(msg, 2);
+			throw (msg);
+		}
+		
 		this._transactionNestingLevel++;
 		this._debug('Transaction in progress -- increasing level to '
 			+ this._transactionNestingLevel, 5);
 	}
 	else {
+		this._transactionWaitLevel = Zotero.waiting;
+		
 		this._debug('Beginning DB transaction', 5);
 		db.beginTransaction();
 		
@@ -481,6 +489,10 @@ Zotero.DBConnection.prototype.commitTransaction = function () {
 		}
 		
 		try {
+			if (!db.transactionInProgress) {
+				throw new Error("No transaction in progress");
+			}
+			
 			db.commitTransaction();
 			
 			if (this.transactionVacuum) {
@@ -630,7 +642,7 @@ Zotero.DBConnection.prototype.getColumns = function (table) {
 		for (var i=0,len=statement.columnCount; i<len; i++) {
 			cols.push(statement.getColumnName(i));
 		}
-		statement.reset();
+		statement.finalize();
 		return cols;
 	}
 	catch (e) {
@@ -692,26 +704,34 @@ Zotero.DBConnection.prototype.getNextID = function (table, column) {
 **/
 Zotero.DBConnection.prototype.getNextName = function (table, field, name)
 {
-	var sql = "SELECT " + field + " FROM " + table + " WHERE " + field
-		+ " LIKE ? ORDER BY " + field + " COLLATE NOCASE";
-	var untitleds = this.columnQuery(sql, name + '%');
-	
-	if (!untitleds || untitleds[0]!=name) {
+	var sql = "SELECT TRIM(SUBSTR(" + field + ", " + (name.length + 1) + ")) "
+				+ "FROM " + table + " "
+				+ "WHERE " + field + " REGEXP '^" + name + "( [0-9]+)?$' "
+				+ "ORDER BY " + field;
+	var suffixes = this.columnQuery(sql);
+	// If none found or first one has a suffix, use default name
+	if (!suffixes || suffixes[0]) {
 		return name;
 	}
 	
+	suffixes.sort(function (a, b) {
+		return parseInt(a) - parseInt(b);
+	});
+	
+	Zotero.debug(suffixes);
+	
 	var i = 1;
+	while (suffixes[i] === "") {
+		i++;
+	}
 	var num = 2;
-	while (untitleds[i] && untitleds[i]==(name + ' ' + num)) {
-		while (untitleds[i+1] && untitleds[i]==untitleds[i+1]) {
-			this._debug('Next ' + i + ' is ' + untitleds[i]);
+	while (suffixes[i] == num) {
+		while (suffixes[i+1] && suffixes[i] == suffixes[i+1]) {
 			i++;
 		}
-		
 		i++;
 		num++;
 	}
-	
 	return name + ' ' + num;
 }
 
@@ -750,7 +770,7 @@ Zotero.DBConnection.prototype.checkException = function (e) {
 		
 		var index = ps.confirmEx(null,
 			Zotero.getString('general.error'),
-			Zotero.getString('db.dbCorrupted', this._dbName) + '\n\n' + Zotero.getString('db.dbCorrupted.restart'),
+			Zotero.getString('db.dbCorrupted', this._dbName) + '\n\n' + Zotero.getString('db.dbCorrupted.restart', Zotero.appName),
 			buttonFlags,
 			Zotero.getString('general.restartNow'),
 			Zotero.getString('general.restartLater'),
@@ -771,8 +791,11 @@ Zotero.DBConnection.prototype.checkException = function (e) {
 
 
 Zotero.DBConnection.prototype.closeDatabase = function () {
-	var db = this._getDBConnection();
-	db.close();
+	if(this._connection) {
+		this.stopDummyStatement();
+		this._connection.close();
+		return true;
+	}
 }
 
 
@@ -851,14 +874,13 @@ Zotero.DBConnection.prototype.backupDatabase = function (suffix) {
 	
 	// Turn off DB locking before backup and reenable after, since otherwise
 	// the lock is lost
-	var dbLockExclusive = Zotero.Prefs.get('dbLockExclusive');
 	var hadDummyStatement = !!this._dummyStatement;
 	try {
-		if (dbLockExclusive) {
-			Zotero.DB.query("PRAGMA locking_mode=NORMAL");
+		if (DB_LOCK_EXCLUSIVE) {
+			this.query("PRAGMA locking_mode=NORMAL");
 		}
 		if (hadDummyStatement) {
-			Zotero.DB.stopDummyStatement();
+			this.stopDummyStatement();
 		}
 		
 		var store = Components.classes["@mozilla.org/storage/service;1"].
@@ -871,11 +893,11 @@ Zotero.DBConnection.prototype.backupDatabase = function (suffix) {
 		return false;
 	}
 	finally {
-		if (dbLockExclusive) {
-			Zotero.DB.query("PRAGMA locking_mode=EXCLUSIVE");
+		if (DB_LOCK_EXCLUSIVE) {
+			this.query("PRAGMA locking_mode=EXCLUSIVE");
 		}
 		if (hadDummyStatement) {
-			Zotero.DB.startDummyStatement();
+			this.startDummyStatement();
 		}
 	}
 	
@@ -1003,8 +1025,10 @@ Zotero.DBConnection.prototype.stopDummyStatement = function () {
 	}
 	
 	Zotero.debug("Stopping dummy statement for '" + this._dbName + "'");
-	this._dummyStatement.reset();
-	this._dummyStatement = null;
+	this._dummyStatement.finalize();
+	this._dummyConnection.close();
+	delete this._dummyConnection;
+	delete this._dummyStatement;
 }
 
 
@@ -1163,9 +1187,7 @@ Zotero.DBConnection.prototype._getDBConnection = function () {
 		throw (e);
 	}
 	
-	// Exclusive locking mode (default) prevents access to Zotero database
-	// while Firefox is open -- normal mode is more convenient for development
-	if (Zotero.Prefs.get('dbLockExclusive')) {
+	if (DB_LOCK_EXCLUSIVE) {
 		Zotero.DB.query("PRAGMA locking_mode=EXCLUSIVE");
 	}
 	else {
