@@ -12,9 +12,10 @@ Zotero.HTTP = new function() {
 	* @param {nsIURI|String}	url				URL to request
 	* @param {Function} 		onDone			Callback to be executed upon request completion
 	* @param {String} 		responseCharset	Character set to force on the response
+	* @param {Zotero.CookieSandbox} [cookieSandbox] Cookie sandbox object
 	* @return {Boolean} True if the request was sent, or false if the browser is offline
 	*/
-	this.doGet = function(url, onDone, responseCharset) {
+	this.doGet = function(url, onDone, responseCharset, cookieSandbox) {
 		if (url instanceof Components.interfaces.nsIURI) {
 			// Don't display password in console
 			var disp = url.clone();
@@ -31,36 +32,23 @@ Zotero.HTTP = new function() {
 			return false;
 		}
 		
-		// Workaround for "Accept third-party cookies" being off in Firefox 3.0.1
-		// https://www.zotero.org/trac/ticket/1070
-		if (Zotero.isFx30) {
-			const Cc = Components.classes;
-			const Ci = Components.interfaces;
-			var ds = Cc["@mozilla.org/webshell;1"].
-						createInstance(Components.interfaces.nsIDocShellTreeItem).
-						QueryInterface(Ci.nsIInterfaceRequestor);
-			ds.itemType = Ci.nsIDocShellTreeItem.typeContent;
-			var xmlhttp = Cc["@mozilla.org/xmlextras/xmlhttprequest;1"].
-							createInstance(Ci.nsIXMLHttpRequest);
-			xmlhttp.mozBackgroundRequest = true;
-			xmlhttp.open("GET", url, true);
-			xmlhttp.channel.loadGroup = ds.getInterface(Ci.nsILoadGroup);
-			xmlhttp.channel.loadFlags |= Ci.nsIChannel.LOAD_DOCUMENT_URI;
-		}
-		else {
-			var xmlhttp = Components.classes["@mozilla.org/xmlextras/xmlhttprequest;1"]
-							.createInstance();
-			// Prevent certificate/authentication dialogs from popping up
-			xmlhttp.mozBackgroundRequest = true;
-			xmlhttp.open('GET', url, true);
-			// Send cookie even if "Allow third-party cookies" is disabled (>=Fx3.6 only)
-			if (!Zotero.isFx35) {
-				var channel = xmlhttp.channel;
-				channel.QueryInterface(Components.interfaces.nsIHttpChannelInternal);
-				channel.forceAllowThirdPartyCookie = true;
-			}
-		}
+		var xmlhttp = Components.classes["@mozilla.org/xmlextras/xmlhttprequest;1"]
+						.createInstance();
 		
+		// Prevent certificate/authentication dialogs from popping up
+		xmlhttp.mozBackgroundRequest = true;
+		xmlhttp.open('GET', url, true);
+		
+		// Send cookie even if "Allow third-party cookies" is disabled (>=Fx3.6 only)
+		var channel = xmlhttp.channel;
+		channel.QueryInterface(Components.interfaces.nsIHttpChannelInternal);
+		channel.forceAllowThirdPartyCookie = true;
+		
+		// Set charset
+		if (responseCharset) {
+			channel.contentCharset = responseCharset;
+		}
+	
 		// Don't cache GET requests
 		xmlhttp.channel.loadFlags |= Components.interfaces.nsIRequest.LOAD_BYPASS_CACHE;
 		
@@ -69,6 +57,7 @@ Zotero.HTTP = new function() {
 			_stateChange(xmlhttp, onDone, responseCharset);
 		};
 		
+		if(cookieSandbox) cookieSandbox.attachToInterfaceRequestor(xmlhttp);
 		xmlhttp.send(null);
 		
 		return xmlhttp;
@@ -82,9 +71,10 @@ Zotero.HTTP = new function() {
 	* @param {Function} onDone Callback to be executed upon request completion
 	* @param {String} headers Request HTTP headers
 	* @param {String} responseCharset Character set to force on the response
+	* @param {Zotero.CookieSandbox} [cookieSandbox] Cookie sandbox object
 	* @return {Boolean} True if the request was sent, or false if the browser is offline
 	*/
-	this.doPost = function(url, body, onDone, headers, responseCharset) {
+	this.doPost = function(url, body, onDone, headers, responseCharset, cookieSandbox) {
 		if (url instanceof Components.interfaces.nsIURI) {
 			// Don't display password in console
 			var disp = url.clone();
@@ -109,34 +99,20 @@ Zotero.HTTP = new function() {
 			return false;
 		}
 		
-		// Workaround for "Accept third-party cookies" being off in Firefox 3.0.1
-		// https://www.zotero.org/trac/ticket/1070
-		if (Zotero.isFx30) {
-			const Cc = Components.classes;
-			const Ci = Components.interfaces;
-			var ds = Cc["@mozilla.org/webshell;1"].
-						createInstance(Components.interfaces.nsIDocShellTreeItem).
-						QueryInterface(Ci.nsIInterfaceRequestor);
-			ds.itemType = Ci.nsIDocShellTreeItem.typeContent;
-			var xmlhttp = Cc["@mozilla.org/xmlextras/xmlhttprequest;1"].
-							createInstance(Ci.nsIXMLHttpRequest);
-			xmlhttp.mozBackgroundRequest = true;
-			xmlhttp.open("POST", url, true);
-			xmlhttp.channel.loadGroup = ds.getInterface(Ci.nsILoadGroup);
-			xmlhttp.channel.loadFlags |= Ci.nsIChannel.LOAD_DOCUMENT_URI;
-		}
-		else {
-			var xmlhttp = Components.classes["@mozilla.org/xmlextras/xmlhttprequest;1"]
-						.createInstance();
-			// Prevent certificate/authentication dialogs from popping up
-			xmlhttp.mozBackgroundRequest = true;
-			xmlhttp.open('POST', url, true);
-			// Send cookie even if "Allow third-party cookies" is disabled (>=Fx3.6 only)
-			if (!Zotero.isFx35) {
-				var channel = xmlhttp.channel;
-				channel.QueryInterface(Components.interfaces.nsIHttpChannelInternal);
-				channel.forceAllowThirdPartyCookie = true;
-			}
+		var xmlhttp = Components.classes["@mozilla.org/xmlextras/xmlhttprequest;1"]
+					.createInstance();
+		// Prevent certificate/authentication dialogs from popping up
+		xmlhttp.mozBackgroundRequest = true;
+		xmlhttp.open('POST', url, true);
+	
+		// Send cookie even if "Allow third-party cookies" is disabled (>=Fx3.6 only)
+		var channel = xmlhttp.channel;
+		channel.QueryInterface(Components.interfaces.nsIHttpChannelInternal);
+		channel.forceAllowThirdPartyCookie = true;
+		
+		// Set charset
+		if (responseCharset) {
+			channel.contentCharset = responseCharset;
 		}
 		
 		if (headers) {
@@ -166,6 +142,7 @@ Zotero.HTTP = new function() {
 			_stateChange(xmlhttp, onDone, responseCharset);
 		};
 		
+		if(cookieSandbox) cookieSandbox.attachToInterfaceRequestor(xmlhttp);
 		xmlhttp.send(body);
 		
 		return xmlhttp;
@@ -177,9 +154,10 @@ Zotero.HTTP = new function() {
 	* @param {String} url URL to request
 	* @param {Function} onDone Callback to be executed upon request completion
 	* @param {Object} requestHeaders HTTP headers to include with request
+	* @param {Zotero.CookieSandbox} [cookieSandbox] Cookie sandbox object
 	* @return {Boolean} True if the request was sent, or false if the browser is offline
 	*/
-	this.doHead = function(url, onDone, requestHeaders) {
+	this.doHead = function(url, onDone, requestHeaders, cookieSandbox) {
 		if (url instanceof Components.interfaces.nsIURI) {
 			// Don't display password in console
 			var disp = url.clone();
@@ -243,6 +221,7 @@ Zotero.HTTP = new function() {
 			_stateChange(xmlhttp, onDone);
 		};
 		
+		if(cookieSandbox) cookieSandbox.attachToInterfaceRequestor(xmlhttp);
 		xmlhttp.send(null);
 		
 		return xmlhttp;
@@ -504,14 +483,23 @@ Zotero.HTTP = new function() {
 	 * @param {Function} processor Callback to be executed for each document loaded
 	 * @param {Function} done Callback to be executed after all documents have been loaded
 	 * @param {Function} exception Callback to be executed if an exception occurs
+	 * @param {Boolean} dontDelete Don't delete the hidden browser upon completion; calling function
+	 *                             must call deleteHiddenBrowser itself.
+	 * @param {Zotero.CookieSandbox} [cookieSandbox] Cookie sandbox object
+	 * @return {browser} Hidden browser used for loading
 	 */
-	this.processDocuments = function(urls, processor, done, exception) {
+	this.processDocuments = function(urls, processor, done, exception, dontDelete, cookieSandbox) {
+		// (Approximately) how many seconds to wait if the document is left in the loading state and
+		// pageshow is called before we call pageshow with an incomplete document
+		const LOADING_STATE_TIMEOUT = 120;
+		
+		var firedLoadEvent;
 		/**
 		 * Removes event listener for the load event and deletes the hidden browser
 		 */
 		var removeListeners = function() {
 			hiddenBrowser.removeEventListener(loadEvent, onLoad, true);
-			Zotero.Browser.deleteHiddenBrowser(hiddenBrowser);
+			if(!dontDelete) Zotero.Browser.deleteHiddenBrowser(hiddenBrowser);
 		}
 		
 		/**
@@ -521,6 +509,7 @@ Zotero.HTTP = new function() {
 		var doLoad = function() {
 			if(urls.length) {
 				var url = urls.shift();
+				firedLoadEvent = 0;
 				try {
 					Zotero.debug("loading "+url);
 					hiddenBrowser.loadURI(url);
@@ -544,12 +533,19 @@ Zotero.HTTP = new function() {
 		 * @inner
 		 */
 		var onLoad = function() {
-			if(hiddenBrowser.contentDocument.location.href == "about:blank") return;
-			Zotero.debug(hiddenBrowser.contentDocument.location.href+" has been loaded");
-			if(hiddenBrowser.contentDocument.location.href != prevUrl) {	// Just in case it fires too many times
-				prevUrl = hiddenBrowser.contentDocument.location.href;
+			var doc = hiddenBrowser.contentDocument,
+				url = doc.location.href.toString();
+			if(url == "about:blank") return;
+			if(doc.readyState === "loading" && firedLoadEvent < 120) {
+				// Try again in a second	
+				firedLoadEvent++;
+				Zotero.setTimeout(onLoad, 1000);
+				return;
+			}
+			if(url !== prevUrl) {	// Just in case it fires too many times
+				prevUrl = url;
 				try {
-					processor(hiddenBrowser.contentDocument);
+					processor(doc);
 				} catch(e) {
 					removeListeners();
 					if(exception) {
@@ -570,8 +566,11 @@ Zotero.HTTP = new function() {
 		
 		var hiddenBrowser = Zotero.Browser.createHiddenBrowser();
 		hiddenBrowser.addEventListener(loadEvent, onLoad, true);
+		if(cookieSandbox) cookieSandbox.attachToBrowser(hiddenBrowser);
 		
 		doLoad();
+		
+		return hiddenBrowser;
 	}
 	
 	/**
