@@ -68,110 +68,160 @@ Zotero.ItemTreeView.prototype._runCallbacks = function() {
 }
 
 
-/*
- *  Called by the tree itself
+/**
+ * Called by the tree itself
  */
 Zotero.ItemTreeView.prototype.setTree = function(treebox)
 {
-	//Zotero.debug("Calling setTree()");
-	// Try to set the window document if not yet set
-	if (treebox && !this._ownerDocument) {
-		try {
-			this._ownerDocument = treebox.treeBody.ownerDocument;
+	var generator = this._setTreeGenerator(treebox);
+	if(generator.next()) {
+		Zotero.pumpGenerator(generator);
+	}
+}
+
+/**
+ * Generator used internally for setting the tree
+ */
+Zotero.ItemTreeView.prototype._setTreeGenerator = function(treebox)
+{
+	try {
+		//Zotero.debug("Calling setTree()");
+		var start = Date.now();
+		// Try to set the window document if not yet set
+		if (treebox && !this._ownerDocument) {
+			try {
+				this._ownerDocument = treebox.treeBody.ownerDocument;
+			}
+			catch (e) {}
 		}
-		catch (e) {}
-	}
-	
-	if (this._treebox) {
-		if (this._needsSort) {
-			this.sort();
+		
+		if (this._treebox) {
+			if (this._needsSort) {
+				this.sort();
+			}
+			yield false;
 		}
-		return;
-	}
-	
-	if (!treebox) {
-		Components.utils.reportError("Passed treebox empty in setTree()");
-	}
-	
-	this._treebox = treebox;
-	
-	if (this._ownerDocument.defaultView.ZoteroPane_Local) {
-		this._ownerDocument.defaultView.ZoteroPane_Local.setItemsPaneMessage(Zotero.getString('pane.items.loading'));
-	}
-	
-	// Generate the tree contents in a timer to allow message above to display
-	var paneLoader = function(obj) {
+		
+		if (!treebox) {
+			Components.utils.reportError("Passed treebox empty in setTree()");
+		}
+		
+		this._treebox = treebox;
+		
+		if (this._ownerDocument.defaultView.ZoteroPane_Local) {
+			this._ownerDocument.defaultView.ZoteroPane_Local.setItemsPaneMessage(Zotero.getString('pane.items.loading'));
+			this._waitAfter = start + 100;
+		}
+		
 		if (Zotero.locked) {
 			var msg = "Zotero is locked -- not loading items tree";
 			Zotero.debug(msg, 2);
 			
-			if (obj._ownerDocument.defaultView.ZoteroPane_Local) {
-				obj._ownerDocument.defaultView.ZoteroPane_Local.clearItemsPaneMessage();
+			if (this._ownerDocument.defaultView.ZoteroPane_Local) {
+				this._ownerDocument.defaultView.ZoteroPane_Local.clearItemsPaneMessage();
 			}
-			return;
+			yield false;
 		}
 		
 		// If a DB transaction is open, display error message and bail
 		if (!Zotero.stateCheck()) {
-			if (obj._ownerDocument.defaultView.ZoteroPane_Local) {
-				obj._ownerDocument.defaultView.ZoteroPane_Local.displayErrorMessage();
+			if (this._ownerDocument.defaultView.ZoteroPane_Local) {
+				this._ownerDocument.defaultView.ZoteroPane_Local.displayErrorMessage();
 			}
-			return;
+			yield false;
 		}
 		
-		obj.refresh();
+		var generator = this._refreshGenerator();
+		while(generator.next()) yield true;
 		
 		// Add a keypress listener for expand/collapse
-		var expandAllRows = obj.expandAllRows;
-		var collapseAllRows = obj.collapseAllRows;
-		var tree = obj._treebox.treeBody.parentNode;
+		var tree = this._treebox.treeBody.parentNode;
+		var me = this;
 		var listener = function(event) {
-			var key = String.fromCharCode(event.which);
-			
-			if (key == '+' && !(event.ctrlKey || event.altKey || event.metaKey)) {
-				obj.expandAllRows(treebox);
+			// Handle arrow keys specially on multiple selection, since
+			// otherwise the tree just applies it to the last-selected row
+			if (event.keyCode == 39 || event.keyCode == 37) {
+				if (me._treebox.view.selection.count > 1) {
+					switch (event.keyCode) {
+						case 39:
+							me.expandSelectedRows();
+							break;
+							
+						case 37:
+							me.collapseSelectedRows();
+							break;
+					}
+					
+					event.preventDefault();
+				}
+				
 				return;
 			}
-			else if (key == '-' && !(event.shiftKey || event.ctrlKey ||
-					event.altKey || event.metaKey)) {
-				obj.collapseAllRows(treebox);
+			
+			var key = String.fromCharCode(event.which);
+			if (key == '+' && !(event.ctrlKey || event.altKey || event.metaKey)) {
+				me.expandAllRows();
+				event.preventDefault();
+				return;
+			}
+			else if (key == '-' && !(event.shiftKey || event.ctrlKey || event.altKey || event.metaKey)) {
+				me.collapseAllRows();
+				event.preventDefault();
 				return;
 			}
 		};
 		// Store listener so we can call removeEventListener()
 		// in overlay.js::onCollectionSelected()
-		obj.listener = listener;
+		this.listener = listener;
 		tree.addEventListener('keypress', listener, false);
 		
-		obj.sort();
-		obj.expandMatchParents();
+		this.sort();
+		
+		// Only yield if there are callbacks; otherwise, we're almost done
+		if(this._callbacks.length && this._waitAfter && Date.now() > this._waitAfter) yield true;
+		
+		this.expandMatchParents();
 		
 		//Zotero.debug('Running callbacks in itemTreeView.setTree()', 4);
-		obj._runCallbacks();
+		this._runCallbacks();
 		
-		if (obj._ownerDocument.defaultView.ZoteroPane_Local) {
-			obj._ownerDocument.defaultView.ZoteroPane_Local.clearItemsPaneMessage();
+		if (this._ownerDocument.defaultView.ZoteroPane_Local) {
+			this._ownerDocument.defaultView.ZoteroPane_Local.clearItemsPaneMessage();
 		}
 		
 		// Select a queued item from selectItem()
-		if (obj._itemGroup && obj._itemGroup.itemToSelect) {
-			var item = obj._itemGroup.itemToSelect;
-			obj.selectItem(item['id'], item['expand']);
-			obj._itemGroup.itemToSelect = null;
+		if (this._itemGroup && this._itemGroup.itemToSelect) {
+			var item = this._itemGroup.itemToSelect;
+			this.selectItem(item['id'], item['expand']);
+			this._itemGroup.itemToSelect = null;
 		}
+		
+		delete this._waitAfter;
+		Zotero.debug("Set tree in "+(Date.now()-start)+" ms");
+	} catch(e) {
+		Zotero.logError(e);
 	}
-	
-	this._ownerDocument.defaultView.setTimeout(paneLoader, 50, this);
+	yield false;
 }
 
-
-/*
+/**
  *  Reload the rows from the data access methods
  *  (doesn't call the tree.invalidate methods, etc.)
  */
 Zotero.ItemTreeView.prototype.refresh = function()
+{
+	var generator = this._refreshGenerator();
+	while(generator.next()) {};
+}
+
+/**
+ * Generator used internally for refresh
+ */
+Zotero.ItemTreeView._haveCachedFields = false;
+Zotero.ItemTreeView.prototype._refreshGenerator = function()
 {
 	Zotero.debug('Refreshing items list');
+	if(!Zotero.ItemTreeView._haveCachedFields) yield true;
 	
 	var usiDisabled = Zotero.UnresponsiveScriptIndicator.disable();
 	
@@ -190,7 +240,7 @@ Zotero.ItemTreeView.prototype.refresh = function()
 	}
 	// If treebox isn't ready, skip refresh
 	catch (e) {
-		return;
+		yield false;
 	}
 	
 	for (var i=0; i<visibleFields.length; i++) {
@@ -205,8 +255,10 @@ Zotero.ItemTreeView.prototype.refresh = function()
 	
 	Zotero.DB.beginTransaction();
 	Zotero.Items.cacheFields(cacheFields);
+	Zotero.ItemTreeView._haveCachedFields = true;
+	
+	var newRows = this._itemGroup.getItems();
 	
-	var newRows = this._itemGroup.getChildItems();
 	var added = 0;
 	
 	for (var i=0, len=newRows.length; i < len; i++) {
@@ -239,6 +291,8 @@ Zotero.ItemTreeView.prototype.refresh = function()
 	}
 	
 	Zotero.DB.commitTransaction();
+		
+	if(this._waitAfter && Date.now() > this._waitAfter) yield true;
 	
 	this._refreshHashMap();
 	
@@ -252,6 +306,8 @@ Zotero.ItemTreeView.prototype.refresh = function()
 	if (usiDisabled) {
 		Zotero.UnresponsiveScriptIndicator.enable();
 	}
+	
+	yield false;
 }
 
 
@@ -276,6 +332,13 @@ Zotero.ItemTreeView.prototype.notify = function(action, type, ids, extraData)
 	var sort = false;
 	
 	var savedSelection = this.saveSelection();
+	var previousRow = false;
+	
+	// Redraw the tree (for tag color changes)
+	if (action == 'redraw') {
+		this._treebox.invalidate();
+		return;
+	}
 	
 	// If refreshing a single item, just unselect and reselect it
 	if (action == 'refresh') {
@@ -297,26 +360,27 @@ Zotero.ItemTreeView.prototype.notify = function(action, type, ids, extraData)
 		return;
 	}
 	
-	if (this._itemGroup.isShare()) {
+	if (itemGroup.isShare()) {
 		return;
 	}
 	
-	this.selection.selectEventsSuppressed = true;
-	
 	// See if we're in the active window
 	var zp = Zotero.getActiveZoteroPane();
 	var activeWindow = zp && zp.itemsView == this;
 	
 	var quicksearch = this._ownerDocument.getElementById('zotero-tb-search');
 	
-	
 	// 'collection-item' ids are in the form collectionID-itemID
 	if (type == 'collection-item') {
+		if (!itemGroup.isCollection()) {
+			return;
+		}
+		
 		var splitIDs = [];
 		for each(var id in ids) {
 			var split = id.split('-');
-			// Skip if not collection or not an item in this collection
-			if (!itemGroup.isCollection() || split[0] != this._itemGroup.ref.id) {
+			// Skip if not an item in this collection
+			if (split[0] != itemGroup.ref.id) {
 				continue;
 			}
 			splitIDs.push(split[1]);
@@ -330,47 +394,58 @@ Zotero.ItemTreeView.prototype.notify = function(action, type, ids, extraData)
 		}
 	}
 	
+	this.selection.selectEventsSuppressed = true;
+	
 	if ((action == 'remove' && !itemGroup.isLibrary(true))
 			|| action == 'delete' || action == 'trash') {
 		
-		// Since a remove involves shifting of rows, we have to do it in order,
-		// so sort the ids by row
-		var rows = [];
-		for(var i=0, len=ids.length; i<len; i++)
-		{
-			if (action == 'delete' || action == 'trash' ||
-					!itemGroup.ref.hasItem(ids[i])) {
-				// Row might already be gone (e.g. if this is a child and
-				// 'modify' was sent to parent)
-				if (this._itemRowMap[ids[i]] != undefined) {
-					rows.push(this._itemRowMap[ids[i]]);
-				}
-			}
-		}
-		
-		if(rows.length > 0)
-		{
-			rows.sort(function(a,b) { return a-b });
-			
-			for(var i=0, len=rows.length; i<len; i++)
-			{
-				var row = rows[i];
-				if(row != null)
-				{
-					this._hideItem(row-i);
-					this._treebox.rowCountChanged(row-i,-1);
-				}
-			}
-			
+		// On a delete in duplicates mode, just refresh rather than figuring
+		// out what to remove
+		if (itemGroup.isDuplicates()) {
+			previousRow = this._itemRowMap[ids[0]];
+			this.refresh();
 			madeChanges = true;
 			sort = true;
 		}
+		else {
+			// Since a remove involves shifting of rows, we have to do it in order,
+			// so sort the ids by row
+			var rows = [];
+			for (var i=0, len=ids.length; i<len; i++) {
+				if (action == 'delete' || action == 'trash' ||
+						!itemGroup.ref.hasItem(ids[i])) {
+					// Row might already be gone (e.g. if this is a child and
+					// 'modify' was sent to parent)
+					if (this._itemRowMap[ids[i]] != undefined) {
+						rows.push(this._itemRowMap[ids[i]]);
+					}
+				}
+			}
+			
+			if (rows.length > 0) {
+				rows.sort(function(a,b) { return a-b });
+				
+				for(var i=0, len=rows.length; i<len; i++)
+				{
+					var row = rows[i];
+					if(row != null)
+					{
+						this._hideItem(row-i);
+						this._treebox.rowCountChanged(row-i,-1);
+					}
+				}
+				
+				madeChanges = true;
+				sort = true;
+			}
+		}
 	}
 	else if (action == 'modify')
 	{
 		// If trash or saved search, just re-run search
 		if (itemGroup.isTrash() || itemGroup.isSearch())
 		{
+			Zotero.ItemGroupCache.clear();
 			this.refresh();
 			madeChanges = true;
 			sort = true;
@@ -429,6 +504,8 @@ Zotero.ItemTreeView.prototype.notify = function(action, type, ids, extraData)
 					if (item.deleted) {
 						continue;
 					}
+					
+					// Otherwise the item has to be added
 					if(item.isRegularItem() || !item.getSource())
 					{
 						//most likely, the note or attachment's parent was removed.
@@ -574,7 +651,9 @@ Zotero.ItemTreeView.prototype.notify = function(action, type, ids, extraData)
 		}
 		else
 		{
-			var previousRow = this._itemRowMap[ids[0]];
+			if (previousRow === false) {
+				previousRow = this._itemRowMap[ids[0]];
+			}
 			
 			if (sort) {
 				this.sort(typeof sort == 'number' ? sort : false);
@@ -583,14 +662,25 @@ Zotero.ItemTreeView.prototype.notify = function(action, type, ids, extraData)
 				this._refreshHashMap();
 			}
 			
-			// On delete, select item at previous position
-			if (action == 'delete' || action == 'remove') {
-				if (this._dataItems[previousRow]) {
-					this.selection.select(previousRow);
+			// On removal of a row, select item at previous position
+			if (action == 'remove' || action == 'trash' || action == 'delete') {
+				// In duplicates view, select the next set on delete
+				if (itemGroup.isDuplicates()) {
+					if (this._dataItems[previousRow]) {
+						// Mirror ZoteroPane.onTreeMouseDown behavior
+						var itemID = this._dataItems[previousRow].ref.id;
+						var setItemIDs = itemGroup.ref.getSetItemsByItemID(itemID);
+						this.selectItems(setItemIDs);
+					}
 				}
-				// If no item at previous position, select last item in list
-				else if (this._dataItems[this._dataItems.length - 1]) {
-					this.selection.select(this._dataItems.length - 1);
+				else {
+					if (this._dataItems[previousRow]) {
+						this.selection.select(previousRow);
+					}
+					// If no item at previous position, select last item in list
+					else if (this._dataItems[this._dataItems.length - 1]) {
+						this.selection.select(this._dataItems.length - 1);
+					}
 				}
 			}
 			else {
@@ -625,7 +715,6 @@ Zotero.ItemTreeView.prototype.unregister = function()
 ////////////////////////////////////////////////////////////////////////////////
 ///
 ///  nsITreeView functions
-///  http://www.xulplanet.com/references/xpcomref/ifaces/nsITreeView.html
 ///
 ////////////////////////////////////////////////////////////////////////////////
 
@@ -670,6 +759,18 @@ Zotero.ItemTreeView.prototype.getCellText = function(row, column)
 		case 'zotero-items-column-accessDate':
 			if (val) {
 				var order = Zotero.Date.getLocaleDateOrder();
+				if (order == 'mdy') {
+					order = 'mdy';
+					var join = '/';
+				}
+				else if (order == 'dmy') {
+					order = 'dmy';
+					var join = '.';
+				}
+				else if (order == 'ymd') {
+					order = 'YMD';
+					var join = '-';
+				}
 				var date = Zotero.Date.sqlToDate(val, true);
 				var parts = [];
 				for (var i=0; i<3; i++) {
@@ -678,16 +779,28 @@ Zotero.ItemTreeView.prototype.getCellText = function(row, column)
 							parts.push(date.getFullYear().toString().substr(2));
 							break;
 							
+						case 'Y':
+							parts.push(date.getFullYear());
+							break;
+						
 						case 'm':
 							parts.push((date.getMonth() + 1));
 							break;
-							
+						
+						case 'M':
+							parts.push(Zotero.Utilities.lpad((date.getMonth() + 1).toString(), '0', 2));
+							break;
+						
 						case 'd':
 							parts.push(date.getDate());
 							break;
+						
+						case 'D':
+							parts.push(Zotero.Utilities.lpad(date.getDate().toString(), '0', 2));
+							break;
 					}
 					
-					val = parts.join('/');
+					val = parts.join(join);
 					val += ' ' + date.toLocaleTimeString();
 				}
 			}
@@ -917,13 +1030,14 @@ Zotero.ItemTreeView.prototype.sort = function(itemID)
 	
 	// Cache primary values while sorting, since base-field-mapped getField()
 	// calls are relatively expensive
-	var cache = [];
+	var cache = {};
 	
 	// Get the display field for a row (which might be a placeholder title)
-	function getField(row) {
-		var field;
-		var type = row.ref.itemTypeID;
-		if (columnField == 'title') {
+	var getField;
+	if (columnField == 'title') {
+		getField = function (row) {
+			var field;
+			var type = row.ref.itemTypeID;
 			switch (type) {
 				case 8: // letter
 				case 10: // interview
@@ -935,24 +1049,23 @@ Zotero.ItemTreeView.prototype.sort = function(itemID)
 					field = row.getField(columnField, unformatted);
 			}
 			// Ignore some leading and trailing characters when sorting
-			field = Zotero.Items.getSortTitle(field);
+			return Zotero.Items.getSortTitle(field);
 		}
-		else {
-			field = row.getField(columnField, unformatted);
-		}
-		return field;
+	} else {
+		getField = function(row) row.getField(columnField, unformatted);
 	}
 	
 	var includeTrashed = this._itemGroup.isTrash();
 	
+	var me = this;
 	function rowSort(a, b) {
 		var cmp, fieldA, fieldB;
 		
-		var aItemID = a.ref.id;
+		var aItemID = a.id;
 		if (cache[aItemID]) {
 			fieldA = cache[aItemID];
 		}
-		var bItemID = b.ref.id;
+		var bItemID = b.id;
 		if (cache[bItemID]) {
 			fieldB = cache[bItemID];
 		}
@@ -1018,14 +1131,14 @@ Zotero.ItemTreeView.prototype.sort = function(itemID)
 				}
 		}
 		
-		if (columnField != 'firstCreator') {
+		if (columnField !== 'firstCreator') {
 			cmp = creatorSort(a, b);
 			if (cmp) {
 				return cmp;
 			}
 		}
 		
-		if (columnField != 'date') {
+		if (columnField !== 'date') {
 			fieldA = a.getField('date', true).substr(0, 10);
 			fieldB = b.getField('date', true).substr(0, 10);
 			
@@ -1046,18 +1159,18 @@ Zotero.ItemTreeView.prototype.sort = function(itemID)
 		//
 		// Try sorting by first word in firstCreator field, since we already have it
 		//
-		var fieldA = firstCreatorSortCache[a.ref.id];
+		var fieldA = firstCreatorSortCache[a.id];
 		if (fieldA == undefined) {
 			var matches = Zotero.Items.getSortTitle(a.getField('firstCreator')).match(/^[^\s]+/);
 			var fieldA = matches ? matches[0] : '';
-			firstCreatorSortCache[a.ref.id] = fieldA;
+			firstCreatorSortCache[a.id] = fieldA;
 		}
 		
-		var fieldB = firstCreatorSortCache[b.ref.id];
+		var fieldB = firstCreatorSortCache[b.id];
 		if (fieldB == undefined) {
 			var matches = Zotero.Items.getSortTitle(b.getField('firstCreator')).match(/^[^\s]+/);
 			var fieldB = matches ? matches[0] : '';
-			firstCreatorSortCache[b.ref.id] = fieldB;
+			firstCreatorSortCache[b.id] = fieldB;
 		}
 		
 		if (!fieldA && !fieldB) {
@@ -1177,6 +1290,15 @@ Zotero.ItemTreeView.prototype.sort = function(itemID)
 			aPos++;
 			var aFound = false;
 			while (aPos < aNumCreators) {
+				// Don't die if there's no creator at an index
+				if (!aCreators[aPos]) {
+					Components.utils.reportError(
+						"Creator is missing at position " + aPos
+						+ " for item " + a.ref.libraryID + "/" + a.ref.key
+					);
+					return -1;
+				}
+				
 				if (aCreators[aPos].creatorTypeID == aFirstCreatorTypeID) {
 					aFound = true;
 					break;
@@ -1187,6 +1309,15 @@ Zotero.ItemTreeView.prototype.sort = function(itemID)
 			bPos++;
 			var bFound = false;
 			while (bPos < bNumCreators) {
+				// Don't die if there's no creator at an index
+				if (!bCreators[bPos]) {
+					Components.utils.reportError(
+						"Creator is missing at position " + bPos
+						+ " for item " + b.ref.libraryID + "/" + b.ref.key
+					);
+					return -1;
+				}
+				
 				if (bCreators[bPos].creatorTypeID == bFirstCreatorTypeID) {
 					bFound = true;
 					break;
@@ -1220,10 +1351,6 @@ Zotero.ItemTreeView.prototype.sort = function(itemID)
 		return (a > b) ? -1 : (a < b) ? 1 : 0;
 	}
 	
-	function reverseSort(a, b) {
-		return rowSort(a,b) * -1;
-	}
-	
 	// Need to close all containers before sorting
 	var openItemIDs = this.saveOpenState(true);
 	
@@ -1231,12 +1358,12 @@ Zotero.ItemTreeView.prototype.sort = function(itemID)
 	if (itemID) {
 		var row = this._itemRowMap[itemID];
 		for (var i=0, len=this._dataItems.length; i<len; i++) {
-			if (i == row) {
+			if (i === row) {
 				continue;
 			}
 			
 			if (order) {
-				var cmp = reverseSort(this._dataItems[i], this._dataItems[row]);
+				var cmp = -1*rowSort(this._dataItems[i], this._dataItems[row]);
 			}
 			else {
 				var cmp = rowSort(this._dataItems[i], this._dataItems[row]);
@@ -1261,12 +1388,8 @@ Zotero.ItemTreeView.prototype.sort = function(itemID)
 	}
 	// Full sort
 	else {
-		if (order) {
-			this._dataItems.sort(rowSort);
-		}
-		else {
-			this._dataItems.sort(reverseSort);
-		}
+		this._dataItems.sort(rowSort);
+		if(!order) this._dataItems.reverse();
 	}
 	
 	this._refreshHashMap();
@@ -1372,6 +1495,41 @@ Zotero.ItemTreeView.prototype.selectItem = function(id, expand, noRecurse)
 	return true;
 }
 
+
+/**
+ * Select multiple top-level items
+ *
+ * @param {Integer[]} ids	An array of itemIDs
+ */
+Zotero.ItemTreeView.prototype.selectItems = function(ids) {
+	if (ids.length == 0) {
+		return;
+	}
+	
+	var rows = [];
+	for each(var id in ids) {
+		rows.push(this._itemRowMap[id]);
+	}
+	rows.sort(function (a, b) {
+		return a - b;
+	});
+	
+	this.selection.clearSelection();
+	
+	this.selection.selectEventsSuppressed = true;
+	
+	var lastStart = 0;
+	for (var i = 0, len = rows.length; i < len; i++) {
+		if (i == len - 1 || rows[i + 1] != rows[i] + 1) {
+			this.selection.rangedSelect(rows[lastStart], rows[i], true);
+			lastStart = i + 1;
+		}
+	}
+	
+	this.selection.selectEventsSuppressed = false;
+}
+
+
 /*
  * Return an array of Item objects for selected items
  *
@@ -1385,7 +1543,7 @@ Zotero.ItemTreeView.prototype.getSelectedItems = function(asIDs)
 		this.selection.getRangeAt(i,start,end);
 		for (var j=start.value; j<=end.value; j++) {
 			if (asIDs) {
-				items.push(this._getItemAtRow(j).ref.id);
+				items.push(this._getItemAtRow(j).id);
 			}
 			else {
 				items.push(this._getItemAtRow(j).ref);
@@ -1429,9 +1587,10 @@ Zotero.ItemTreeView.prototype.deleteSelection = function (force)
 	{
 		this.selection.getRangeAt(i,start,end);
 		for (var j=start.value; j<=end.value; j++)
-			ids.push(this._getItemAtRow(j).ref.id);
+			ids.push(this._getItemAtRow(j).id);
 	}
 	
+	Zotero.ItemGroupCache.clear();
 	var itemGroup = this._itemGroup;
 	
 	if (itemGroup.isBucket()) {
@@ -1440,10 +1599,7 @@ Zotero.ItemTreeView.prototype.deleteSelection = function (force)
 	else if (itemGroup.isTrash()) {
 		Zotero.Items.erase(ids);
 	}
-	else if (itemGroup.isGroup() || (force && itemGroup.isWithinGroup())) {
-		Zotero.Items.erase(ids);
-	}
-	else if (itemGroup.isLibrary() || force) {
+	else if (itemGroup.isLibrary(true) || force) {
 		Zotero.Items.trash(ids);
 	}
 	else if (itemGroup.isCollection()) {
@@ -1696,7 +1852,8 @@ Zotero.ItemTreeView.prototype.rememberFirstRow = function(firstRow) {
 }
 
 
-Zotero.ItemTreeView.prototype.expandAllRows = function(treebox) {
+Zotero.ItemTreeView.prototype.expandAllRows = function() {
+	this.selection.selectEventsSuppressed = true;
 	this._treebox.beginUpdateBatch();
 	for (var i=0; i<this.rowCount; i++) {
 		if (this.isContainer(i) && !this.isContainerOpen(i)) {
@@ -1705,10 +1862,11 @@ Zotero.ItemTreeView.prototype.expandAllRows = function(treebox) {
 	}
 	this._refreshHashMap();
 	this._treebox.endUpdateBatch();
+	this.selection.selectEventsSuppressed = false;
 }
 
 
-Zotero.ItemTreeView.prototype.collapseAllRows = function(treebox) {
+Zotero.ItemTreeView.prototype.collapseAllRows = function() {
 	this._treebox.beginUpdateBatch();
 	for (var i=0; i<this.rowCount; i++) {
 		if (this.isContainer(i) && this.isContainerOpen(i)) {
@@ -1720,6 +1878,38 @@ Zotero.ItemTreeView.prototype.collapseAllRows = function(treebox) {
 }
 
 
+Zotero.ItemTreeView.prototype.expandSelectedRows = function() {
+	var start = {}, end = {};
+	this._treebox.beginUpdateBatch();
+	for (var i = 0, len = this.selection.getRangeCount(); i<len; i++) {
+		this.selection.getRangeAt(i, start, end);
+		for (var j = start.value; j <= end.value; j++) {
+			if (this.isContainer(j) && !this.isContainerOpen(j)) {
+				this.toggleOpenState(j, true);
+			}
+		}
+	}
+	this._refreshHashMap();
+	this._treebox.endUpdateBatch();
+}
+
+
+Zotero.ItemTreeView.prototype.collapseSelectedRows = function() {
+	var start = {}, end = {};
+	this._treebox.beginUpdateBatch();
+	for (var i = 0, len = this.selection.getRangeCount(); i<len; i++) {
+		this.selection.getRangeAt(i, start, end);
+		for (var j = start.value; j <= end.value; j++) {
+			if (this.isContainer(j) && this.isContainerOpen(j)) {
+				this.toggleOpenState(j, true);
+			}
+		}
+	}
+	this._refreshHashMap();
+	this._treebox.endUpdateBatch();
+}
+
+
 Zotero.ItemTreeView.prototype.getVisibleFields = function() {
 	var columns = [];
 	for (var i=0, len=this._treebox.columns.count; i<len; i++) {
@@ -1888,7 +2078,7 @@ Zotero.ItemTreeView.prototype.onDragStart = function (event) {
 	}
 	
 	// Get Quick Copy format for current URL
-	var url = this._ownerDocument.defaultView.content ?
+	var url = this._ownerDocument.defaultView.content && this._ownerDocument.defaultView.content.location ?
 				this._ownerDocument.defaultView.content.location.href : null;
 	var format = Zotero.QuickCopy.getFormatFromURL(url);
 	
@@ -2221,6 +2411,34 @@ Zotero.ItemTreeView.prototype.canDrop = function(row, orient, dragData)
 					return false;
 				}
 				
+				// Don't allow children to be dragged within their own parents
+				var parentItemID = item.getSource();
+				var parentIndex = this._itemRowMap[parentItemID];
+				if (this.getLevel(row) > 0) {
+					if (this._getItemAtRow(this.getParentIndex(row)).ref.id == parentItemID) {
+						return false;
+					}
+				}
+				// Including immediately after the parent
+				if (orient == 1) {
+					if (row == parentIndex) {
+						return false;
+					}
+				}
+				// And immediately before the next parent
+				if (orient == -1) {
+					var nextParentIndex = null;
+					for (var i = parentIndex + 1; i < this.rowCount; i++) {
+						if (this.getLevel(i) == 0) {
+							nextParentIndex = i;
+							break;
+						}
+					}
+					if (row === nextParentIndex) {
+						return false;
+					}
+				}
+				
 				// Disallow cross-library child drag
 				if (item.libraryID != itemGroup.ref.libraryID) {
 					return false;
@@ -2446,12 +2664,39 @@ Zotero.ItemTreeView.prototype.onDragExit = function (event) {
 ////////////////////////////////////////////////////////////////////////////////
 
 Zotero.ItemTreeView.prototype.isSeparator = function(row) 						{ return false; }
-Zotero.ItemTreeView.prototype.getRowProperties = function(row, prop) { }
-Zotero.ItemTreeView.prototype.getColumnProperties = function(col, prop) 		{ }
-
-/* Mark items not matching search as context rows, displayed in gray */
+Zotero.ItemTreeView.prototype.getRowProperties = function(row, prop) {
+	if (!this.selection.isSelected(row)) {
+		return;
+	}
+	
+	var itemID = this._getItemAtRow(row).ref.id;
+	
+	// Set background color for selected items with colored tags
+	if (color = Zotero.Tags.getItemColor(itemID)) {
+		var aServ = Components.classes["@mozilla.org/atom-service;1"].
+			getService(Components.interfaces.nsIAtomService);
+		prop.AppendElement(aServ.getAtom("color" + color.substr(1)));
+	}
+}
+Zotero.ItemTreeView.prototype.getColumnProperties = function(col, prop) { }
 Zotero.ItemTreeView.prototype.getCellProperties = function(row, col, prop) {
-	if (this._searchMode && !this._searchItemIDs[this._getItemAtRow(row).ref.id]) {
+	var itemID = this._getItemAtRow(row).ref.id;
+	
+	// Set tag colors
+	//
+	// Don't set the text color if the row is selected, in which case the background
+	// color is set in getRowProperties() instead, unless the tree isn't focused,
+	// in which case it's not
+	if (!this.selection.isSelected(row) || !this._treebox.focused) {
+		if (color = Zotero.Tags.getItemColor(itemID)) {
+			var aServ = Components.classes["@mozilla.org/atom-service;1"].
+				getService(Components.interfaces.nsIAtomService);
+			prop.AppendElement(aServ.getAtom("color" + color.substr(1)));
+		}
+	}
+	
+	// Mark items not matching search as context rows, displayed in gray
+	if (this._searchMode && !this._searchItemIDs[itemID]) {
 		var aServ = Components.classes["@mozilla.org/atom-service;1"].
 			getService(Components.interfaces.nsIAtomService);
 		prop.AppendElement(aServ.getAtom("contextRow"));
@@ -2463,6 +2708,7 @@ Zotero.ItemTreeView.TreeRow = function(ref, level, isOpen)
 	this.ref = ref;			//the item associated with this
 	this.level = level;
 	this.isOpen = isOpen;
+	this.id = ref.id;
 }
 
 Zotero.ItemTreeView.TreeRow.prototype.getField = function(field, unformatted)
