@@ -202,7 +202,8 @@ Zotero.LocateManager = new function() {
 			"image/png":"png",
 			"image/jpeg":"jpg",
 			"image/gif":"gif",
-			"image/x-icon":"ico"
+			"image/x-icon":"ico",
+			"image/vnd.microsoft.icon":"ico"
 		};
 		
 		// ensure there is an icon
@@ -269,7 +270,7 @@ Zotero.LocateManager = new function() {
 			var ns = engine._urlNamespaces[nsPrefix];
 			if(!ns) return false;
 		} else {
-			if(param === "searchTerms") return [item.getField("title")];
+			if(param === "searchTerms") return [item.title];
 			return false;
 		}
 		
@@ -277,7 +278,7 @@ Zotero.LocateManager = new function() {
 			// take a normal "title," even though we don't use it, because it is valid (but not
 			// preferred) OpenURL
 			if(param === "title") {
-				var title = item.getField("title");
+				var title = item.title;
 				return (title ? [encodeURIComponent(title)] : false);
 			}
 			
@@ -298,7 +299,7 @@ Zotero.LocateManager = new function() {
 			} else if(param === "year") {
 				return (itemOpenURL["rft.date"] ? [itemOpenURL["rft.date"][0].substr(0, 4)] : false);
 			} else {
-				var result = item.getField(param);
+				var result = item[param];
 				return (result ? [encodeURIComponent(result)] : false);
 			}
 		} else {
@@ -412,6 +413,10 @@ Zotero.LocateManager = new function() {
 			if(responseType && responseType !== "text/html") {
 				throw "LocateManager supports only responseType text/html";
 			}
+		
+			if(item.toArray) {
+				item = item.toArray();
+			}
 			
 			var itemAsOpenURL = Zotero.OpenURL.createContextObject(item, "1.0", true);
 			
@@ -514,7 +519,7 @@ Zotero.LocateManager = new function() {
 				false)
 			  return this;
 			
-			throw Cr.NS_ERROR_NO_INTERFACE;
+			throw Components.results.NS_ERROR_NO_INTERFACE;
 		},
 		
 		// nsIRequestObserver
