@@ -343,6 +343,13 @@ Zotero.Search.prototype.save = function(fixGaps) {
 			Zotero.DB.query(sql, sqlParams);
 		}
 		
+		
+		if (isNew && this.libraryID) {
+			var groupID = Zotero.Groups.getGroupIDFromLibraryID(this.libraryID);
+			var group = Zotero.Groups.get(groupID);
+			group.clearSearchCache();
+		}
+		
 		Zotero.DB.commitTransaction();
 	}
 	catch (e) {
@@ -398,29 +405,43 @@ Zotero.Search.prototype.addCondition = function(condition, operator, value, requ
 	}
 	
 	// Shortcut to add a condition on every table -- does not return an id
-	if (condition=='quicksearch'){
+	if (condition.match(/^quicksearch/)) {
 		var parts = Zotero.SearchConditions.parseSearchString(value);
 		
 		for each(var part in parts) {
 			this.addCondition('blockStart');
-			this.addCondition('field', operator, part.text, false);
-			this.addCondition('creator', operator, part.text, false);
-			this.addCondition('tag', operator, part.text, false);
-			this.addCondition('note', operator, part.text, false);
-			this.addCondition('annotation', operator, part.text, false);
-			
-			if (part.inQuotes) {
-				this.addCondition('fulltextContent', operator, part.text, false);
+			if (condition == 'quicksearch-titleCreatorYear') {
+				this.addCondition('title', operator, part.text, false);
+				this.addCondition('year', operator, part.text, false);
 			}
 			else {
-				var splits = Zotero.Fulltext.semanticSplitter(part.text);
-				for each(var split in splits) {
-					this.addCondition('fulltextWord', operator, split, false);
+				this.addCondition('field', operator, part.text, false);
+				this.addCondition('tag', operator, part.text, false);
+				this.addCondition('note', operator, part.text, false);
+			}
+			this.addCondition('creator', operator, part.text, false);
+			
+			if (condition == 'quicksearch-everything') {
+				this.addCondition('annotation', operator, part.text, false);
+				
+				if (part.inQuotes) {
+					this.addCondition('fulltextContent', operator, part.text, false);
+				}
+				else {
+					var splits = Zotero.Fulltext.semanticSplitter(part.text);
+					for each(var split in splits) {
+						this.addCondition('fulltextWord', operator, split, false);
+					}
 				}
 			}
 			
 			this.addCondition('blockEnd');
 		}
+		
+		if (condition == 'quicksearch-titleCreatorYear') {
+			this.addCondition('noChildren', 'true');
+		}
+		
 		return false;
 	}
 	// Shortcut to add a collection
@@ -631,7 +652,7 @@ Zotero.Search.prototype.search = function(asTempTable){
 				return false;
 			}
 			
-			var tmpTable = this._idsToTempTable(ids);
+			var tmpTable = Zotero.Search.idsToTempTable(ids);
 		}
 		// Otherwise, just copy to temp table directly
 		else {
@@ -644,7 +665,7 @@ Zotero.Search.prototype.search = function(asTempTable){
 		}
 		
 		// Search ids in temp table
-		var sql = "SELECT itemID FROM items WHERE itemID IN (" + this._sql + ") "
+		var sql = "SELECT GROUP_CONCAT(itemID) FROM items WHERE itemID IN (" + this._sql + ") "
 			+ "AND ("
 			+ "itemID IN (SELECT itemID FROM " + tmpTable + ")";
 		
@@ -656,7 +677,8 @@ Zotero.Search.prototype.search = function(asTempTable){
 		}
 		sql += ")";
 		
-		var ids = Zotero.DB.columnQuery(sql, this._sqlParams);
+		var res = Zotero.DB.valueQuery(sql, this._sqlParams),
+			ids = res ? res.split(",") : [];
 		/*
 		// DEBUG: Should this be here?
 		//
@@ -700,12 +722,13 @@ Zotero.Search.prototype.search = function(asTempTable){
 				if (joinMode == 'any') {
 					if (!tmpTable) {
 						Zotero.DB.beginTransaction();
-						var tmpTable = this._idsToTempTable(ids);
+						var tmpTable = Zotero.Search.idsToTempTable(ids);
 					}
 					
-					var sql = "SELECT itemID FROM items WHERE "
+					var sql = "SELECT GROUP_CONCAT(itemID) FROM items WHERE "
 						+ "itemID NOT IN (SELECT itemID FROM " + tmpTable + ")";
-					var scopeIDs = Zotero.DB.columnQuery(sql);
+					var res = Zotero.DB.valueQuery(sql).split(",");
+					var scopeIDs = res ? res.split(",") : [];
 				}
 				// If an ALL search, scan only items from the main search
 				else {
@@ -819,7 +842,7 @@ Zotero.Search.prototype.search = function(asTempTable){
 	if (this.hasPostSearchFilter() &&
 			(includeParentsAndChildren || includeParents || includeChildren)) {
 		Zotero.DB.beginTransaction();
-		var tmpTable = this._idsToTempTable(ids);
+		var tmpTable = Zotero.Search.idsToTempTable(ids);
 		
 		if (includeParentsAndChildren || includeParents) {
 			//Zotero.debug("Adding parent items to result set");
@@ -846,8 +869,9 @@ Zotero.Search.prototype.search = function(asTempTable){
 			}
 		}
 		
-		sql = "SELECT itemID FROM items WHERE itemID IN (" + sql + ")";
-		var parentChildIDs = Zotero.DB.columnQuery(sql);
+		sql = "SELECT GROUP_CONCAT(itemID) FROM items WHERE itemID IN (" + sql + ")";
+		var res = Zotero.DB.valueQuery(sql);
+		var parentChildIDs = res ? res.split(",") : [];
 		Zotero.DB.query("DROP TABLE " + tmpTable);
 		Zotero.DB.commitTransaction();
 		
@@ -870,7 +894,8 @@ Zotero.Search.prototype.search = function(asTempTable){
 	}
 	
 	if (asTempTable) {
-		return this._idsToTempTable(ids);
+		var table = Zotero.Search.idsToTempTable(ids);
+		return table;
 	}
 	
 	return ids;
@@ -931,17 +956,27 @@ Zotero.Search.prototype._prepFieldChange = function (field) {
 /*
  * Batch insert
  */
-Zotero.Search.prototype._idsToTempTable = function (ids) {
+Zotero.Search.idsToTempTable = function (ids) {
+	const N_COMBINED_INSERTS = 128;
+	
 	var tmpTable = "tmpSearchResults_" + Zotero.randomString(8);
 	
 	Zotero.DB.beginTransaction();
 	
 	var sql = "CREATE TEMPORARY TABLE " + tmpTable + " (itemID INT)";
 	Zotero.DB.query(sql);
-	var sql = "INSERT INTO " + tmpTable + " VALUES (?)";
-	var insertStatement = Zotero.DB.getStatement(sql);
-	for (var i=0; i<ids.length; i++) {
-		insertStatement.bindInt32Parameter(0, ids[i]);
+	
+	var sql = "INSERT INTO " + tmpTable + " SELECT ? ";
+	for (var i=0; i<(N_COMBINED_INSERTS-1); i++) {
+		sql += "UNION SELECT ? ";
+	}
+	
+	var insertStatement = Zotero.DB.getStatement(sql),
+		n = ids.length;
+	for (var i=0; i<n; i+=N_COMBINED_INSERTS) {
+		for (var j=0; j<N_COMBINED_INSERTS; j++) {
+			insertStatement.bindInt32Parameter(j, ids[i+j]);
+		}
 		try {
 			insertStatement.execute();
 		}
@@ -949,7 +984,19 @@ Zotero.Search.prototype._idsToTempTable = function (ids) {
 			throw (Zotero.DB.getLastErrorString());
 		}
 	}
-	insertStatement.reset();
+	insertStatement.finalize();
+	
+	var nRemainingInserts = (n % N_COMBINED_INSERTS);
+	var remainingInsertStart = n-nRemainingInserts-1;
+	var sql = "INSERT INTO " + tmpTable + " SELECT ? ";
+	for (var i=remainingInsertStart; i<n; i++) {
+		sql += "UNION SELECT ? ";
+	}
+	
+	var insertStatement = Zotero.DB.getStatement(sql);
+	for (var j=remainingInsertStart; j<n; j++) {
+		insertStatement.bindInt32Parameter(j-remainingInsertStart, ids[remainingInsertStart]);
+	}
 	
 	var sql = "CREATE INDEX " + tmpTable + "_itemID ON " + tmpTable + "(itemID)";
 	Zotero.DB.query(sql);
@@ -976,7 +1023,7 @@ Zotero.Search.prototype._buildQuery = function(){
 		var data = Zotero.SearchConditions.get(this._conditions[i]['condition']);
 		
 		// Has a table (or 'savedSearch', which doesn't have a table but isn't special)
-		if (data.table || data.name == 'savedSearch') {
+		if (data.table || data.name == 'savedSearch' || data.name == 'tempTable') {
 			conditions.push({
 				name: data['name'],
 				alias: data['name']!=this._conditions[i]['condition']
@@ -1020,7 +1067,7 @@ Zotero.Search.prototype._buildQuery = function(){
 					var unfiled = this._conditions[i]['operator'] == 'true';
 					continue;
 				
-				// Search subfolders
+				// Search subcollections
 				case 'recursive':
 					var recursive = this._conditions[i]['operator']=='true';
 					continue;
@@ -1143,6 +1190,15 @@ Zotero.Search.prototype._buildQuery = function(){
 						openParens++;
 						break;
 					
+					case 'year':
+						condSQL += 'fieldID IN (?) AND ';
+						condSQLParams.push(Zotero.ItemFields.getID('date'));
+						condSQL += "valueID IN (SELECT valueID FROM "
+							+ "itemDataValues WHERE ";
+						
+						openParens++;
+						break;
+					
 					case 'collection':
 						var col;
 						if (condition.value) {
@@ -1275,6 +1331,14 @@ Zotero.Search.prototype._buildQuery = function(){
 						openParens++;
 						break;
 					
+					case 'tempTable':
+						if (!condition.value.match(/^[a-zA-Z0-9]+$/)) {
+							throw ("Invalid temp table '" + condition.value + "'");
+						}
+						condSQL += "itemID IN (SELECT id FROM " + condition.value + ")";
+						skipOperators = true;
+						break;
+						
 					// For quicksearch blocks
 					case 'blockStart':
 					case 'blockEnd':
@@ -1817,6 +1881,40 @@ Zotero.SearchConditions = new function(){
 				}
 			},
 			
+			{
+				name: 'quicksearch-titleCreatorYear',
+				operators: {
+					is: true,
+					isNot: true,
+					contains: true,
+					doesNotContain: true
+				},
+				noLoad: true
+			},
+			
+			{
+				name: 'quicksearch-fields',
+				operators: {
+					is: true,
+					isNot: true,
+					contains: true,
+					doesNotContain: true
+				},
+				noLoad: true
+			},
+			
+			{
+				name: 'quicksearch-everything',
+				operators: {
+					is: true,
+					isNot: true,
+					contains: true,
+					doesNotContain: true
+				},
+				noLoad: true
+			},
+			
+			// Deprecated
 			{
 				name: 'quicksearch',
 				operators: {
@@ -2041,6 +2139,19 @@ Zotero.SearchConditions = new function(){
 				template: true // mark for special handling
 			},
 			
+			{
+				name: 'year',
+				operators: {
+					is: true,
+					isNot: true,
+					contains: true,
+					doesNotContain: true
+				},
+				table: 'itemData',
+				field: 'SUBSTR(value, 1, 4)',
+				special: true
+			},
+			
 			{
 				name: 'numberfield',
 				operators: {
@@ -2100,6 +2211,13 @@ Zotero.SearchConditions = new function(){
 					doesNotContain: true
 				},
 				special: false
+			},
+			
+			{
+				name: 'tempTable',
+				operators: {
+					is: true
+				}
 			}
 		];
 		
