@@ -417,12 +417,20 @@ function ChromeExtensionHandler() {
 							valA = Zotero.Items.getSortTitle(valA);
 							valB = Zotero.Items.getSortTitle(valB);
 						}
-						// Use multipart date -- would be better just to get
-						// the unformatted date value directly, but toArray(),
-						// which we currently use above, doesn't do that
 						else if (sorts[index].field == 'date') {
-							valA = Zotero.Date.strToMultipart(a[sorts[index].field]);
-							valB = Zotero.Date.strToMultipart(b[sorts[index].field]);
+							var itemA = Zotero.Items.getByLibraryAndKey(a.libraryID, a.key);
+							var itemB = Zotero.Items.getByLibraryAndKey(b.libraryID, b.key);
+							valA = itemA.getField('date', true, true);
+							valB = itemB.getField('date', true, true);
+						}
+						// TEMP: This is an ugly hack to make creator sorting
+						// slightly less broken. To do this right, real creator
+						// sorting needs to be abstracted from itemTreeView.js.
+						else if (sorts[index].field == 'firstCreator') {
+							var itemA = Zotero.Items.getByLibraryAndKey(a.libraryID, a.key);
+							var itemB = Zotero.Items.getByLibraryAndKey(b.libraryID, b.key);
+							valA = itemA.getField('firstCreator');
+							valB = itemB.getField('firstCreator');
 						}
 						else {
 							valA = a[sorts[index].field];
@@ -852,13 +860,21 @@ function ChromeExtensionHandler() {
 				var [path, queryString] = uri.path.substr(1).split('?');
 				var [type, id] = path.split('/');
 				
-				//currently only able to select one item
+				// currently only able to select one item
 				var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"]
 					.getService(Components.interfaces.nsIWindowMediator);
-				var win = wm.getMostRecentWindow(null);
+				var win = wm.getMostRecentWindow("navigator:browser");
 				
+				// restore window if it's in the dock
+				if(win.windowState == Components.interfaces.nsIDOMChromeWindow.STATE_MINIMIZED) {
+					win.restore();
+				}
+				
+				// open Zotero pane
 				win.ZoteroPane.show();
 				
+				if(!id) return;
+				
 				var lkh = Zotero.Items.parseLibraryKeyHash(id);
 				if (lkh) {
 					var item = Zotero.Items.getByLibraryAndKey(lkh.libraryID, lkh.key);
@@ -898,7 +914,7 @@ function ChromeExtensionHandler() {
 			generateContent: try {
 				var window = Components.classes["@mozilla.org/embedcomp/window-watcher;1"]
 					.getService(Components.interfaces.nsIWindowWatcher)
-					.openWindow(null, 'chrome://zotero/content/standalone.xul', '',
+					.openWindow(null, 'chrome://zotero/content/standalone/standalone.xul', '',
 						'chrome,centerscreen,resizable', null);
 			}
 			catch (e) {
@@ -959,7 +975,7 @@ function ChromeExtensionHandler() {
 	
 	ConnectorChannel.prototype.contentCharset = "UTF-8";
 	ConnectorChannel.prototype.contentType = "text/html";
-	ConnectorChannel.prototype.notificationCallbacks = {};
+	ConnectorChannel.prototype.notificationCallbacks = null;
 	ConnectorChannel.prototype.securityInfo = null;
 	ConnectorChannel.prototype.status = 0;
 	ConnectorChannel.prototype.loadGroup = null;
@@ -970,10 +986,12 @@ function ChromeExtensionHandler() {
 	ConnectorChannel.prototype.__defineSetter__("originalURI", function() { });
 	
 	ConnectorChannel.prototype.asyncOpen = function(streamListener, context) {
+		if(this.loadGroup) this.loadGroup.addRequest(this, null);
 		streamListener.onStartRequest(this, context);
 		streamListener.onDataAvailable(this, context, this._stream, 0, this.contentLength);
 		streamListener.onStopRequest(this, context, this.status);
 		this._isPending = false;
+		if(this.loadGroup) this.loadGroup.removeRequest(this, null, 0);
 	}
 	
 	ConnectorChannel.prototype.isPending = function() {
@@ -986,13 +1004,9 @@ function ChromeExtensionHandler() {
 		if(this._stream) this._stream.close();
 	}
 	
-	ConnectorChannel.prototype.suspend = function(status) {
-		this.status = status;
-	}
+	ConnectorChannel.prototype.suspend = function() {}
 	
-	ConnectorChannel.prototype.resume = function(status) {
-		this.status = status;
-	}
+	ConnectorChannel.prototype.resume = function() {}
 	
 	ConnectorChannel.prototype.open = function() {
 		return this._stream;
@@ -1026,10 +1040,10 @@ function ChromeExtensionHandler() {
 			try {
 				var originalURI = uri.path;
 				originalURI = decodeURIComponent(originalURI.substr(originalURI.indexOf("/")+1));
-				if(!Zotero.Connector.Data[originalURI]) {
+				if(!Zotero.Server.Connector.Data[originalURI]) {
 					return null;
 				} else {
-					return new ConnectorChannel(originalURI, Zotero.Connector.Data[originalURI]);
+					return new ConnectorChannel(originalURI, Zotero.Server.Connector.Data[originalURI]);
 				}
 			} catch(e) {
 				Zotero.debug(e);
